# This file is not meant for public use and will be removed v0.8.0.
# Use the `quantecon` namespace for importing the objects
# included below.

import warnings
from . import _graph_tools


__all__ = ['DiGraph', 'random_tournament_graph', 'annotate_nodes']


def __dir__():
    return __all__


def __getattr__(name):
    if name not in __all__:
        raise AttributeError(
                "`quantecon.graph_tools` is deprecated and has no attribute "
                f"'{name}'."
            )

    warnings.warn(f"Please use `{name}` from the `quantecon` namespace, the"
                  "`quantecon.graph_tools` namespace is deprecated. You can use"
                  f" the following instead:\n `from quantecon import {name}`.",
                  category=DeprecationWarning, stacklevel=2)

    return getattr(_graph_tools, name)



"""
Tests for game_theory/utilities.py

"""
import numpy as np
from numpy.testing import assert_array_equal
from quantecon.game_theory.utilities import (
    _copy_action_to, _copy_action_profile_to
)


def test_copy_action_to():
    pure_action = 1
    n = 3
    mixed_action = np.zeros(n)
    mixed_action[pure_action] = 1
    dst = np.empty(n)
    _copy_action_to(dst, pure_action)
    assert_array_equal(dst, mixed_action)

    mixed_action = np.array([0.2, 0.3, 0.5])
    n = len(mixed_action)
    dst = np.empty(n)
    _copy_action_to(dst, mixed_action)
    assert_array_equal(dst, mixed_action)


def test_copy_action_profile_to():
    pure_action = 1
    n0 = 3
    mixed_action0 = np.zeros(n0)
    mixed_action0[pure_action] = 1

    action_profile = (pure_action, np.array([0.2, 0.3, 0.5]), [0.4, 0.6])
    mixed_actions = (mixed_action0,) + action_profile[1:]
    nums_actions = tuple(len(x) for x in mixed_actions)
    dst = tuple(np.empty(n) for n in nums_actions)

    _copy_action_profile_to(dst, mixed_actions)
    for x, y in zip(dst, mixed_actions):
        assert_array_equal(x, y)


if __name__ == '__main__':
    import sys
    import nose

    argv = sys.argv[:]
    argv.append('--verbose')
    argv.append('--nocapture')
    nose.main(argv=argv, defaultTest=__file__)



# flake8: noqa
"""
API for QuantEcon Utilities
"""

from .array import searchsorted
from .notebooks import fetch_nb_dependencies
from .random import check_random_state, rng_integers
from .timing import tic, tac, toc, loop_timer



"""
Tests for support_enumeration.py

"""
import numpy as np
from numpy.testing import assert_allclose, assert_, assert_raises
from quantecon.util import check_random_state
from quantecon.game_theory import Player, NormalFormGame, support_enumeration


def random_skew_sym(n, m=None, random_state=None):
    """
    Generate a random skew symmetric zero-sum NormalFormGame of the form
    O    B
    -B.T O
    where B is an n x m matrix.

    """
    if m is None:
        m = n
    random_state = check_random_state(random_state)
    B = random_state.random((n, m))
    A = np.empty((n+m, n+m))
    A[:n, :n] = 0
    A[n:, n:] = 0
    A[:n, n:] = B
    A[n:, :n] = -B.T
    return NormalFormGame([Player(A) for i in range(2)])


class TestSupportEnumeration():
    def setup_method(self):
        self.game_dicts = []

        # From von Stengel 2007 in Algorithmic Game Theory
        bimatrix = [[(3, 3), (3, 2)],
                    [(2, 2), (5, 6)],
                    [(0, 3), (6, 1)]]
        d = {'g': NormalFormGame(bimatrix),
             'NEs': [([1, 0, 0], [1, 0]),
                     ([4/5, 1/5, 0], [2/3, 1/3]),
                     ([0, 1/3, 2/3], [1/3, 2/3])]}
        self.game_dicts.append(d)

        # Degenerate game
        # NEs ([0, p, 1-p], [1/2, 1/2]), 0 <= p <= 1, are not detected.
        bimatrix = [[(1, 1), (-1, 0)],
                    [(-1, 0), (1, 0)],
                    [(0, 0), (0, 0)]]
        d = {'g': NormalFormGame(bimatrix),
             'NEs': [([1, 0, 0], [1, 0]),
                     ([0, 1, 0], [0, 1])]}
        self.game_dicts.append(d)

    def test_support_enumeration(self):
        for d in self.game_dicts:
            NEs_computed = support_enumeration(d['g'])
            assert_(len(NEs_computed) == len(d['NEs']))
            for actions_computed, actions in zip(NEs_computed, d['NEs']):
                for action_computed, action in zip(actions_computed, actions):
                    assert_allclose(action_computed, action)

    def test_no_error_skew_sym(self):
        # Test no LinAlgError is raised.
        n, m = 3, 2
        seed = 7028
        g = random_skew_sym(n, m, random_state=seed)
        support_enumeration(g)


def test_support_enumeration_invalid_g():
    bimatrix = [[(3, 3), (3, 2)],
                [(2, 2), (5, 6)],
                [(0, 3), (6, 1)]]
    assert_raises(TypeError, support_enumeration, bimatrix)



"""
Implements the empirical cumulative distribution function given an array
of observations.

"""

import numpy as np


class ECDF:
    """
    One-dimensional empirical distribution function given a vector of
    observations.

    Parameters
    ----------
    observations : array_like
        An array of observations

    Attributes
    ----------
    observations : see Parameters

    """

    def __init__(self, observations):
        self.observations = np.asarray(observations)

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        m = "Empirical CDF:\n  - number of observations: {n}"
        return m.format(n=self.observations.size)

    def __call__(self, x):
        """
        Evaluates the ecdf at x

        Parameters
        ----------
        x : scalar(float)
            The x at which the ecdf is evaluated

        Returns
        -------
        scalar(float)
            Fraction of the sample less than x

        """
        def f(a):
            return np.mean(self.observations <= a)
        vf = np.frompyfunc(f, 1, 1)
        return vf(x).astype(float)



"""
Tests for distributions.py

"""
import numpy as np
from numpy.testing import assert_allclose, assert_
from math import sqrt
from quantecon.distributions import BetaBinomial


class TestBetaBinomial:
    def setup_method(self):
        self.n = 100
        self.a = 5
        self.b = 5
        self.test_obj = BetaBinomial(self.n, self.a, self.b)

    def test_init(self):
        assert_(self.test_obj.n == self.n)
        assert_(self.test_obj.a == self.a)
        assert_(self.test_obj.b == self.b)

    def test_mean(self):
        assert_(self.test_obj.mean == 50)

    def test_std(self):
        assert_(self.test_obj.std == sqrt(250))

    def test_var(self):
        assert_(self.test_obj.var == 250)

    def test_skew(self):
        assert_(self.test_obj.skew == 0)

    def test_pdf(self):
        n = 9
        a = 1
        b = 1
        test_obj = BetaBinomial(n, a, b)
        assert_allclose(test_obj.pdf(), np.full(n+1, 1/(n+1)))



"""
Tests for mclennan_tourky.py
"""
import numpy as np
from numpy.testing import assert_array_equal, assert_raises, assert_
from quantecon.game_theory import Player, NormalFormGame, mclennan_tourky
from quantecon.game_theory.mclennan_tourky import (
    _best_response_selection, _flatten_action_profile, _is_epsilon_nash
)


class TestMclennanTourky():
    def setup_method(self):
        def anti_coordination(N, v):
            payoff_array = np.empty((2,)*N)
            payoff_array[0, :] = 1
            payoff_array[1, :] = 0
            payoff_array[1].flat[0] = v
            g = NormalFormGame((Player(payoff_array),)*N)
            return g

        def p_star(N, v):
            # Unique symmetric NE mixed action: [p_star, 1-p_star]
            return 1 / (v**(1/(N-1)))

        def epsilon_nash_interval(N, v, epsilon):
            # Necessary, but not sufficient, condition: lb < p < ub
            lb = p_star(N, v) - epsilon / ((N-1)*(v**(1/(N-1))-1))
            ub = p_star(N, v) + epsilon / (N-1)
            return lb, ub

        self.game_dicts = []
        v = 2
        epsilon = 1e-5

        Ns = [2, 3, 4]
        for N in Ns:
            g = anti_coordination(N, v)
            lb, ub = epsilon_nash_interval(N, v, epsilon)
            d = {'g': g,
                 'epsilon': epsilon,
                 'lb': lb,
                 'ub': ub}
            self.game_dicts.append(d)

    def test_convergence_default(self):
        for d in self.game_dicts:
            NE, res = mclennan_tourky(d['g'], full_output=True)
            assert_(res.converged)

    def test_pure_nash(self):
        for d in self.game_dicts:
            init = (1,) + (0,)*(d['g'].N-1)
            NE, res = mclennan_tourky(d['g'], init=init, full_output=True)
            assert_(res.num_iter == 1)


class TestMclennanTourkyInvalidInputs():
    def setup_method(self):
            self.bimatrix = [[(3, 3), (3, 2)],
                             [(2, 2), (5, 6)],
                             [(0, 3), (6, 1)]]
            self.g = NormalFormGame(self.bimatrix)

    def test_mclennan_tourky_invalid_g(self):
        assert_raises(TypeError, mclennan_tourky, self.bimatrix)

    def test_mclennan_tourky_invalid_init_type(self):
        assert_raises(TypeError, mclennan_tourky, self.g, 1)

    def test_mclennan_tourky_invalid_init_length(self):
        assert_raises(ValueError, mclennan_tourky, self.g, [1])


class TestEpsilonNash():
    def setup_method(self):
        def anti_coordination(N, v):
            payoff_array = np.empty((2,)*N)
            payoff_array[0, :] = 1
            payoff_array[1, :] = 0
            payoff_array[1].flat[0] = v
            g = NormalFormGame((Player(payoff_array),)*N)
            return g

        def p_star(N, v):
            # Unique symmetric NE mixed action: [p_star, 1-p_star]
            return 1 / (v**(1/(N-1)))

        def epsilon_nash_interval(N, v, epsilon):
            # Necessary, but not sufficient, condition: lb < p < ub
            lb = p_star(N, v) - epsilon / ((N-1)*(v**(1/(N-1))-1))
            ub = p_star(N, v) + epsilon / (N-1)
            return lb, ub

        self.game_dicts = []
        v = 2
        epsilon = 1e-5

        Ns = [2, 3, 4]
        for N in Ns:
            g = anti_coordination(N, v)
            lb, ub = epsilon_nash_interval(N, v, epsilon)
            d = {'g': g,
                 'epsilon': epsilon,
                 'lb': lb,
                 'ub': ub}
            self.game_dicts.append(d)

        self.bimatrix = [[(3, 3), (3, 2)],
                         [(2, 2), (5, 6)],
                         [(0, 3), (6, 1)]]
        self.g = NormalFormGame(self.bimatrix)

    def test_epsilon_nash_with_full_output(self):
        for d in self.game_dicts:
            NE, res = \
                mclennan_tourky(d['g'], epsilon=d['epsilon'], full_output=True)
            for i in range(d['g'].N):
                assert_(d['lb'] < NE[i][0] < d['ub'])

    def test_epsilon_nash_without_full_output(self):
        for d in self.game_dicts:
            NE = mclennan_tourky(d['g'], epsilon=d['epsilon'],
                                 full_output=False)
            for i in range(d['g'].N):
                assert_(d['lb'] < NE[i][0] < d['ub'])

    def test_is_epsilon_nash_no_indptr(self):
        assert_(_is_epsilon_nash([1., 0., 0., 1., 0.], self.g, 1e-5))


def test_flatten_action_profile():
    unflattened_actions = [[1/3, 1/3, 1/3], [1/2, 1/2]]
    flattened_actions = [1/3, 1/3, 1/3, 1/2, 1/2]
    test_obj = _flatten_action_profile(unflattened_actions, [0, 3, 5])
    assert_array_equal(test_obj, flattened_actions)


def test_best_response_selection_no_indptr():
    bimatrix = [[(3, 3), (3, 2)],
                [(2, 2), (5, 6)],
                [(0, 3), (6, 1)]]
    g = NormalFormGame(bimatrix)

    test_obj = _best_response_selection([1/3, 1/3, 1/3, 1/2, 1/2], g)
    expected_output = np.array([0., 1., 0., 0., 1.])

    assert_array_equal(test_obj, expected_output)



import numpy as np
from .normal_form_game import NormalFormGame
from ..util import check_random_state
from .random import _random_mixed_actions
from .utilities import _copy_action_profile_to


class FictitiousPlay:
    """
    Class representing a fictitious play model.

    Parameters
    ----------
    data : NormalFormGame, or array_like
        The game played in a fictitious play model. `data` must be either
        `NormalFormGame` or an array. See `NormalFormGame`.

    gain : scalar(float), optional(default=None)
        The gain of fictitous play model. If gain is None, the model becomes a
        decreasing gain model. If gain is a scalar, the model becomes a
        constant gain model.

    Attributes
    ----------
    g : NomalFormGame
        The game played in the model.

    N : scalar(int)
        The number of players in the model.

    players : tuple(Player)
        Tuple of the Player instances in the model.

    nums_actions : tuple(int)
        Tuple of the number of actions, one for each player.

    """
    def __init__(self, data, gain=None):
        if isinstance(data, NormalFormGame):
            self.g = data
        else:
            self.g = NormalFormGame(data)

        self.N = self.g.N
        self.players = self.g.players
        self.nums_actions = self.g.nums_actions
        self.tie_breaking = 'smallest'

        if gain is None:
            self.step_size = lambda t: 1 / (t+2)  # decreasing gain
        else:
            self.step_size = lambda t: gain  # constant gain

    def _play(self, actions, t, brs, tie_breaking, tol, random_state):
        for i, player in enumerate(self.players):
            opponents_actions = tuple(actions[i+1:]) + tuple(actions[:i])
            brs[i] = player.best_response(
                opponents_actions if self.N > 2 else opponents_actions[0],
                tie_breaking=tie_breaking, tol=tol, random_state=random_state
            )

        for i in range(self.N):
            actions[i][:] *= 1 - self.step_size(t)
            actions[i][brs[i]] += self.step_size(t)

        return actions

    def play(self, actions=None, num_reps=1, t_init=0, out=None, **options):
        """
        Return a new action profile which is updated by playing the game
        `num_reps` times.

        Parameters
        ----------
        actions : tuple(array_like(float)), optional(default=None)
            The action profile in the initial period. If None, selected
            randomly.

        num_reps : scalar(int), optional(default=1)
            The number of iterations.

        t_init : scalar(int), optional(default=0)
            The period when the game starts.

        out : tuple(array_like(float)), optional(default=None)
            Alternative output tuple of arrays in which to place the result.
            Must be of the same shape as the expected output.

        **options : Keyword arguments passed to the best response method and
                    other methods.

        Returns
        -------
        tuple(ndarray(float, ndim=1))
            The mixed action profile after iteration.

        """
        tie_breaking = options.get('tie_breaking', self.tie_breaking)
        tol = options.get('tol', None)
        random_state = check_random_state(options.get('random_state', None))

        if out is None:
            out = tuple(np.empty(n) for n in self.nums_actions)

        if actions is None:
            _random_mixed_actions(out, random_state)
        else:
            _copy_action_profile_to(out, actions)

        brs = np.empty(self.N, dtype=int)
        for t in range(t_init, t_init+num_reps):
            out = self._play(out, t, brs, tie_breaking, tol, random_state)

        return out

    def time_series(self, ts_length, init_actions=None, t_init=0, **options):
        """
        Return a tuple of arrays representing a time series of mixed action
        profiles.

        Parameters
        ----------
        ts_length : scalar(int)
            The number of iterations.

        init_actions : tuple(int), optional(default=None)
            The action profile in the initial period. If None, selected
            randomly.

        t_init : scalar(int), optional(default=0)
            The period when the game starts.

        **options : Keyword arguments passed to the best response method and
                    other methods.

        Returns
        -------
        tuple(ndarray(float, ndim=2))
            Tuple of arrays representing time series of mixed action profile.

        """
        tie_breaking = options.get('tie_breaking', self.tie_breaking)
        tol = options.get('tol', None)
        random_state = check_random_state(options.get('random_state', None))

        out = tuple(np.empty((ts_length, n)) for n in self.nums_actions)
        out_init = tuple(out[i][0, :] for i in range(self.N))

        if init_actions is None:
            _random_mixed_actions(out_init, random_state)
        else:
            _copy_action_profile_to(out_init, init_actions)

        actions = tuple(np.copy(action) for action in out_init)
        brs = np.empty(self.N, dtype=int)
        for j in range(1, ts_length):
            self._play(actions, t_init+j-1, brs,
                       tie_breaking, tol, random_state)
            for i in range(self.N):
                np.copyto(out[i][j, :], actions[i])

        return out


class StochasticFictitiousPlay(FictitiousPlay):
    """
    Class representing a stochastic fictitious play model.

    Parameters
    ----------
    data : NormalFormGame or array_like
        The game played in the stochastic fictitious play model.

    distribution : scipy.stats object
        The distribution of payoff shocks, which is a `scipy.stats` object.

    gain : scalar(scalar), optional(default=None)
        The gain of fictitious play model. If gain is None, the model becomes a
        decreasing gain model. If gain is a scalar, the model becomes a
        constant gain model.

    Attributes
    ----------
    See attributes of `FictitousPlay`.

    """
    def __init__(self, data, distribution, gain=None):
        FictitiousPlay.__init__(self, data, gain)

        self.payoff_perturbation_dist = \
            lambda size, random_state: distribution.rvs(
                                        size=size,
                                        random_state=random_state)

    def _play(self, actions, t, brs, tie_breaking, tol, random_state):
        for i, player in enumerate(self.players):
            opponents_actions = tuple(actions[i+1:]) + tuple(actions[:i])
            payoff_perturbation = \
                self.payoff_perturbation_dist(size=self.nums_actions[i],
                                              random_state=random_state)
            brs[i] = player.best_response(
                opponents_actions if self.N > 2 else opponents_actions[0],
                tie_breaking=tie_breaking,
                payoff_perturbation=payoff_perturbation,
                tol=tol, random_state=random_state
            )

        for i in range(self.N):
            actions[i][:] *= 1 - self.step_size(t)
            actions[i][brs[i]] += self.step_size(t)

        return actions



"""
This module provides functions to compute quadratic sums of the form described
in the docstrings.

"""


import numpy as np
import scipy.linalg
from ._matrix_eqn import solve_discrete_lyapunov


def var_quadratic_sum(A, C, H, beta, x0):
    r"""
    Computes the expected discounted quadratic sum

    .. math::

        q(x_0) = \mathbb{E} \Big[ \sum_{t=0}^{\infty} \beta^t x_t' H x_t \Big]

    Here :math:`{x_t}` is the VAR process :math:`x_{t+1} = A x_t + C w_t`
    with :math:`{x_t}` standard normal and :math:`x_0` the initial condition.

    Parameters
    ----------
    A : array_like(float, ndim=2)
        The matrix described above in description.  Should be n x n
    C : array_like(float, ndim=2)
        The matrix described above in description.  Should be n x n
    H : array_like(float, ndim=2)
        The matrix described above in description.  Should be n x n
    beta: scalar(float)
        Should take a value in (0, 1)
    x_0: array_like(float, ndim=1)
        The initial condtion. A conformable array (of length n, or with
        n rows)

    Returns
    -------
    q0: scalar(float)
        Represents the value :math:`q(x_0)`

    Remarks: The formula for computing :math:`q(x_0)` is
    :math:`q(x_0) = x_0' Q x_0 + v`
    where

        * :math:`Q` is the solution to :math:`Q = H + \beta A' Q A`, and
        * :math:`v = \frac{trace(C' Q C) \beta}{(1 - \beta)}`

    """
    # == Make sure that A, C, H and x0 are array_like == #

    A, C, H = list(map(np.atleast_2d, (A, C, H)))
    x0 = np.atleast_1d(x0)
    # == Start computations == #
    Q = scipy.linalg.solve_discrete_lyapunov(np.sqrt(beta) * A.T, H)
    cq = np.dot(np.dot(C.T, Q), C)
    v = np.trace(cq) * beta / (1 - beta)
    q0 = np.dot(np.dot(x0.T, Q), x0) + v

    return q0


def m_quadratic_sum(A, B, max_it=50):
    r"""
    Computes the quadratic sum

    .. math::

        V = \sum_{j=0}^{\infty} A^j B A^{j'}

    V is computed by solving the corresponding discrete lyapunov
    equation using the doubling algorithm.  See the documentation of
    `util.solve_discrete_lyapunov` for more information.

    Parameters
    ----------
    A : array_like(float, ndim=2)
        An n x n matrix as described above.  We assume in order for
        convergence that the eigenvalues of :math:`A` have moduli bounded by
        unity
    B : array_like(float, ndim=2)
        An n x n matrix as described above.  We assume in order for
        convergence that the eigenvalues of :math:`A` have moduli bounded by
        unity
    max_it : scalar(int), optional(default=50)
        The maximum number of iterations

    Returns
    -------
    gamma1: array_like(float, ndim=2)
        Represents the value :math:`V`

    """

    gamma1 = solve_discrete_lyapunov(A, B, max_it)

    return gamma1



"""
Filename: test_brd.py

Tests for brd.py

"""
import numpy as np
from numpy.testing import assert_array_equal

from quantecon.game_theory import BRD, KMR, SamplingBRD


class TestBRD:
    '''Test the methods of BRD'''

    def setup_method(self):
        '''Setup a BRD instance'''
        # 2x2 coordination game with action 1 risk-dominant
        payoff_matrix = [[4, 0],
                         [3, 2]]
        self.N = 4  # 4 players
        self.brd = BRD(payoff_matrix, self.N)

    def test_time_series_1(self):
        assert_array_equal(
            self.brd.time_series(ts_length=3, init_action_dist=[4, 0]),
            [[4, 0],
             [4, 0],
             [4, 0]]
            )

    def test_time_series_2(self):
        seed = 329478856717593533176523622896549543480
        x = [self.brd.time_series(ts_length=3, init_action_dist=[2, 2],
                                  random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_equal(x[0], x[1])


class TestKMR:
    '''Test the methods of KMR'''

    def setup_method(self):
        payoff_matrix = [[4, 0],
                         [3, 2]]
        self.N = 4
        self.kmr = KMR(payoff_matrix, self.N)

    def test_time_series(self):
        seed = 21519527815966711149598801341951879349
        x = [self.kmr.time_series(ts_length=3, init_action_dist=[2, 2],
                                  random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_equal(x[0], x[1])


class TestSamplingBRD:
    '''Test the methods of SamplingBRD'''

    def setup_method(self):
        payoff_matrix = [[4, 0],
                         [3, 2]]
        self.N = 4
        self.sbrd = SamplingBRD(payoff_matrix, self.N)

    def test_time_series(self):
        seed = 205165126657120054758393970887343077472
        x = [self.sbrd.time_series(ts_length=3, init_action_dist=[2, 2],
                                   random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_equal(x[0], x[1])



"""
Tests for graph_tools.py

"""

import numpy as np
from numpy.testing import assert_array_equal, assert_raises, assert_

from quantecon import DiGraph, random_tournament_graph


def list_of_array_equal(s, t):
    """
    Compare two lists of ndarrays

    s, t: lists of numpy.ndarrays

    """
    assert_(len(s) == len(t))
    all(assert_array_equal(x, y) for x, y in zip(s, t))


class Graphs:
    """Setup graphs for the tests"""

    def __init__(self):
        self.strongly_connected_graph_dicts = []
        self.not_strongly_connected_graph_dicts = []

        graph_dict = {
            'A': np.array([[1, 0], [0, 1]]),
            'strongly_connected_components':
            [np.array([0]), np.array([1])],
            'sink_strongly_connected_components':
            [np.array([0]), np.array([1])],
            'is_strongly_connected': False,
        }
        self.not_strongly_connected_graph_dicts.append(graph_dict)

        graph_dict = {
            'A': np.array([[1, 0, 0], [1, 0, 1], [0, 0, 1]]),
            'strongly_connected_components':
            [np.array([0]), np.array([1]), np.array([2])],
            'sink_strongly_connected_components':
            [np.array([0]), np.array([2])],
            'is_strongly_connected': False,
        }
        self.not_strongly_connected_graph_dicts.append(graph_dict)

        graph_dict = {
            'A': np.array([[1, 1], [1, 1]]),
            'strongly_connected_components': [np.arange(2)],
            'sink_strongly_connected_components': [np.arange(2)],
            'is_strongly_connected': True,
            'period': 1,
            'is_aperiodic': True,
            'cyclic_components': [np.arange(2)],
        }
        self.strongly_connected_graph_dicts.append(graph_dict)

        graph_dict = {
            'A': np.array([[0, 1], [1, 0]]),
            'strongly_connected_components': [np.arange(2)],
            'sink_strongly_connected_components': [np.arange(2)],
            'is_strongly_connected': True,
            'period': 2,
            'is_aperiodic': False,
            'cyclic_components': [np.array([0]), np.array([1])],
        }
        self.strongly_connected_graph_dicts.append(graph_dict)

        graph_dict = {
            'A': np.array([[0, 1, 0, 0],
                           [0, 0, 1, 0],
                           [1, 0, 0, 1],
                           [0, 0, 1, 0]]),
            'strongly_connected_components': [np.arange(4)],
            'sink_strongly_connected_components': [np.arange(4)],
            'is_strongly_connected': True,
            'period': 1,
            'is_aperiodic': True,
            'cyclic_components': [np.arange(4)],
        }
        self.strongly_connected_graph_dicts.append(graph_dict)

        # Weighted graph
        graph_dict = {
            'A': np.array([[0, 0.5], [2, 0]]),
            'weighted': True,
            'strongly_connected_components': [np.arange(2)],
            'sink_strongly_connected_components': [np.arange(2)],
            'is_strongly_connected': True,
            'period': 2,
            'is_aperiodic': False,
            'cyclic_components': [np.array([0]), np.array([1])],
        }
        self.strongly_connected_graph_dicts.append(graph_dict)

        # Degenrate graph with no edge
        graph_dict = {
            'A': np.array([[0]]),
            'strongly_connected_components': [np.arange(1)],
            'sink_strongly_connected_components': [np.arange(1)],
            'is_strongly_connected': True,
            'period': 1,
            'is_aperiodic': True,
            'cyclic_components': [np.array([0])],
        }
        self.strongly_connected_graph_dicts.append(graph_dict)

        # Degenrate graph with self loop
        graph_dict = {
            'A': np.array([[1]]),
            'strongly_connected_components': [np.arange(1)],
            'sink_strongly_connected_components': [np.arange(1)],
            'is_strongly_connected': True,
            'period': 1,
            'is_aperiodic': True,
            'cyclic_components': [np.array([0])],
        }
        self.strongly_connected_graph_dicts.append(graph_dict)

        self.graph_dicts = \
            self.strongly_connected_graph_dicts + \
            self.not_strongly_connected_graph_dicts


class TestDiGraph:
    """Test the methods in Digraph"""

    def setup_method(self):
        """Setup Digraph instances"""
        self.graphs = Graphs()
        for graph_dict in self.graphs.graph_dicts:
            try:
                weighted = graph_dict['weighted']
            except:
                weighted = False
            graph_dict['g'] = DiGraph(graph_dict['A'], weighted=weighted)

    def test_strongly_connected_components(self):
        for graph_dict in self.graphs.graph_dicts:
            list_of_array_equal(
                sorted(graph_dict['g'].strongly_connected_components,
                       key=lambda x: x[0]),
                sorted(graph_dict['strongly_connected_components'],
                       key=lambda x: x[0])
            )

    def test_num_strongly_connected_components(self):
        for graph_dict in self.graphs.graph_dicts:
            assert_(graph_dict['g'].num_strongly_connected_components ==
                    len(graph_dict['strongly_connected_components']))

    def test_sink_strongly_connected_components(self):
        for graph_dict in self.graphs.graph_dicts:
            list_of_array_equal(
                sorted(graph_dict['g'].sink_strongly_connected_components,
                       key=lambda x: x[0]),
                sorted(graph_dict['sink_strongly_connected_components'],
                       key=lambda x: x[0])
            )

    def test_num_sink_strongly_connected_components(self):
        for graph_dict in self.graphs.graph_dicts:
            assert_(graph_dict['g'].num_sink_strongly_connected_components ==
                    len(graph_dict['sink_strongly_connected_components']))

    def test_is_strongly_connected(self):
        for graph_dict in self.graphs.graph_dicts:
            assert_(graph_dict['g'].is_strongly_connected ==
                    graph_dict['is_strongly_connected'])

    def test_period(self):
        for graph_dict in self.graphs.graph_dicts:
            try:
                assert_(graph_dict['g'].period == graph_dict['period'])
            except NotImplementedError:
                assert_(not graph_dict['g'].is_strongly_connected)

    def test_is_aperiodic(self):
        for graph_dict in self.graphs.graph_dicts:
            try:
                assert_(graph_dict['g'].is_aperiodic ==
                        graph_dict['is_aperiodic'])
            except NotImplementedError:
                assert_(not graph_dict['g'].is_strongly_connected)

    def test_cyclic_components(self):
        for graph_dict in self.graphs.graph_dicts:
            try:
                list_of_array_equal(
                    sorted(graph_dict['g'].cyclic_components,
                           key=lambda x: x[0]),
                    sorted(graph_dict['cyclic_components'],
                           key=lambda x: x[0])
                )
            except NotImplementedError:
                assert_(not graph_dict['g'].is_strongly_connected)


def test_subgraph():
    adj_matrix = [[0, 1, 0], [0, 0, 1], [1, 0, 0]]
    g = DiGraph(adj_matrix)
    nodes = [1, 2]

    subgraph_adj_matrix = [[False, True], [False, False]]
    assert_array_equal(
        g.subgraph(nodes).csgraph.toarray(),
        subgraph_adj_matrix
    )


def test_subgraph_weighted():
    adj_matrix = np.arange(3**2).reshape(3, 3)
    g = DiGraph(adj_matrix, weighted=True)
    nodes = [0, 1]

    subgraph_adj_matrix = [[0, 1], [3, 4]]
    assert_array_equal(
        g.subgraph(nodes).csgraph.toarray(),
        subgraph_adj_matrix
    )


def test_node_labels_connected_components():
    adj_matrix = [[1, 0, 0], [1, 0, 0], [0, 0, 1]]
    node_labels = np.array(['a', 'b', 'c'])
    g = DiGraph(adj_matrix, node_labels=node_labels)

    sccs = [[0], [1], [2]]
    sink_sccs = [[0], [2]]

    properties = ['strongly_connected_components',
                  'sink_strongly_connected_components']
    suffix = '_indices'
    for prop0, components_ind in zip(properties, [sccs, sink_sccs]):
        for return_indices in [True, False]:
            if return_indices:
                components = components_ind
                prop = prop0 + suffix
            else:
                components = [node_labels[i] for i in components_ind]
                prop = prop0
            list_of_array_equal(
                sorted(getattr(g, prop), key=lambda x: x[0]),
                sorted(components, key=lambda x: x[0])
            )


def test_node_labels_cyclic_components():
    adj_matrix = [[0, 1], [1, 0]]
    node_labels = np.array(['a', 'b'])
    g = DiGraph(adj_matrix, node_labels=node_labels)

    cyclic_components = [[0], [1]]

    properties = ['cyclic_components']
    suffix = '_indices'
    for prop0, components_ind in zip(properties, [cyclic_components]):
        for return_indices in [True, False]:
            if return_indices:
                components = components_ind
                prop = prop0 + suffix
            else:
                components = [node_labels[i] for i in components_ind]
                prop = prop0
            list_of_array_equal(
                sorted(getattr(g, prop), key=lambda x: x[0]),
                sorted(components, key=lambda x: x[0])
            )


def test_node_labels_subgraph():
    adj_matrix = [[0, 1, 0], [0, 0, 1], [1, 0, 0]]
    node_labels = np.array(['a', 'b', 'c'])
    g = DiGraph(adj_matrix, node_labels=node_labels)
    nodes = [1, 2]

    assert_array_equal(
        g.subgraph(nodes).node_labels,
        node_labels[nodes]
    )


def test_raises_value_error_non_sym():
    """Test with non symmetric input"""
    assert_raises(ValueError, DiGraph, np.array([[0.4, 0.6]]))


def test_raises_non_homogeneous_node_labels():
    adj_matrix = [[1, 0], [0, 1]]
    node_labels = [(0, 1), 2]
    assert_raises(ValueError, DiGraph, adj_matrix, node_labels=node_labels)


class TestRandomTournamentGraph:
    def setup_method(self):
        n = 5
        g = random_tournament_graph(n)
        self.adj_matrix = g.csgraph.toarray()
        self.eye_bool = np.eye(n, dtype=bool)

    def test_diagonal(self):
        # Test no self loop
        assert_(not self.adj_matrix[self.eye_bool].any())

    def test_off_diagonal(self):
        # Test for each pair of distinct nodes to have exactly one edge
        assert_((self.adj_matrix ^ self.adj_matrix.T)[~self.eye_bool].all())


def test_random_tournament_graph_seed():
    n = 7
    seed = 1234
    graphs = [random_tournament_graph(n, random_state=seed) for i in range(2)]
    assert_array_equal(*[g.csgraph.toarray() for g in graphs])



"""
Contain a Numba-jitted linear programming solver based on the Simplex
Method.

"""
from collections import namedtuple
import numpy as np
from numba import jit
from .pivoting import _pivoting, _lex_min_ratio_test


SimplexResult = namedtuple(
    'SimplexResult', ['x', 'lambd', 'fun', 'success', 'status', 'num_iter']
)
SimplexResult.__doc__ = \
    'namedtuple containing the result from `linprog_simplex`.'

FEA_TOL = 1e-6
TOL_PIV = 1e-7
TOL_RATIO_DIFF = 1e-13

PivOptions = namedtuple(
    'PivOptions', ['fea_tol', 'tol_piv', 'tol_ratio_diff']
)
PivOptions.__new__.__defaults__ = (FEA_TOL, TOL_PIV, TOL_RATIO_DIFF)
PivOptions.__doc__ = 'namedtuple to hold tolerance values for pivoting.'


# Delete useless docstring for fields of namedtuple
def _del_field_docstring(nt):
    for field in nt._fields:
        getattr(nt, field).__doc__ = ''


for _nt in [SimplexResult, PivOptions]:
    _del_field_docstring(_nt)


@jit(nopython=True, cache=True)
def linprog_simplex(c, A_ub=np.empty((0, 0)), b_ub=np.empty((0,)),
                    A_eq=np.empty((0, 0)), b_eq=np.empty((0,)),
                    max_iter=10**6, piv_options=PivOptions(),
                    tableau=None, basis=None, x=None, lambd=None):
    """
    Solve a linear program in the following form by the simplex
    algorithm (with the lexicographic pivoting rule):

    maximize::

        c @ x

    subject to::

        A_ub @ x <= b_ub
        A_eq @ x == b_eq
               x >= 0

    Parameters
    ----------
    c : ndarray(float, ndim=1)
        ndarray of shape (n,).

    A_ub : ndarray(float, ndim=2), optional
        ndarray of shape (m, n).

    b_ub : ndarray(float, ndim=1), optional
        ndarray of shape (m,).

    A_eq : ndarray(float, ndim=2), optional
        ndarray of shape (k, n).

    b_eq : ndarray(float, ndim=1), optional
        ndarray of shape (k,).

    max_iter : int, optional(default=10**6)
        Maximum number of iteration to perform.

    piv_options : PivOptions, optional
        PivOptions namedtuple to set the following tolerance values:

            fea_tol : float
                Feasibility tolerance (default={FEA_TOL}).

            tol_piv : float
                Pivot tolerance (default={TOL_PIV}).

            tol_ratio_diff : float
                Tolerance used in the the lexicographic pivoting
                (default={TOL_RATIO_DIFF}).

    tableau : ndarray(float, ndim=2), optional
        Temporary ndarray of shape (L+1, n+m+L+1) to store the tableau,
        where L=m+k. Modified in place.

    basis : ndarray(int, ndim=1), optional
        Temporary ndarray of shape (L,) to store the basic variables.
        Modified in place.

    x : ndarray(float, ndim=1), optional
        Output ndarray of shape (n,) to store the primal solution.

    lambd : ndarray(float, ndim=1), optional
        Output ndarray of shape (L,) to store the dual solution.

    Returns
    -------
    res : SimplexResult
        namedtuple consisting of the fields:

            x : ndarray(float, ndim=1)
                ndarray of shape (n,) containing the primal solution.

            lambd : ndarray(float, ndim=1)
                ndarray of shape (L,) containing the dual solution.

            fun : float
                Value of the objective function.

            success : bool
                True if the algorithm succeeded in finding an optimal
                solution.

            status : int
                An integer representing the exit status of the
                optimization::

                    0 : Optimization terminated successfully
                    1 : Iteration limit reached
                    2 : Problem appears to be infeasible
                    3 : Problem apperas to be unbounded

            num_iter : int
                The number of iterations performed.

    References
    ----------
    * K. C. Border, "The Gauss–Jordan and Simplex Algorithms" 2004.

    """
    n, m, k = c.shape[0], A_ub.shape[0], A_eq.shape[0]
    L = m + k

    if tableau is None:
        tableau = np.empty((L+1, n+m+L+1))
    if basis is None:
        basis = np.empty(L, dtype=np.int_)
    if x is None:
        x = np.empty(n)
    if lambd is None:
        lambd = np.empty(L)

    num_iter = 0
    fun = -np.inf

    b_signs = np.empty(L, dtype=np.bool_)
    for i in range(m):
        b_signs[i] = True if b_ub[i] >= 0 else False
    for i in range(k):
        b_signs[m+i] = True if b_eq[i] >= 0 else False

    # Construct initial tableau for Phase 1
    _initialize_tableau(A_ub, b_ub, A_eq, b_eq, tableau, basis)

    # Phase 1
    success, status, num_iter_1 = \
        solve_tableau(tableau, basis, max_iter, skip_aux=False,
                      piv_options=piv_options)
    num_iter += num_iter_1
    if not success:  # max_iter exceeded
        return SimplexResult(x, lambd, fun, success, status, num_iter)
    if tableau[-1, -1] > piv_options.fea_tol:  # Infeasible
        success = False
        status = 2
        return SimplexResult(x, lambd, fun, success, status, num_iter)

    # Modify the criterion row for Phase 2
    _set_criterion_row(c, basis, tableau)

    # Phase 2
    success, status, num_iter_2 = \
        solve_tableau(tableau, basis, max_iter-num_iter, skip_aux=True,
                      piv_options=piv_options)
    num_iter += num_iter_2
    fun = get_solution(tableau, basis, x, lambd, b_signs)

    return SimplexResult(x, lambd, fun, success, status, num_iter)


linprog_simplex.__doc__ = linprog_simplex.__doc__.format(
    FEA_TOL=FEA_TOL, TOL_PIV=TOL_PIV, TOL_RATIO_DIFF=TOL_RATIO_DIFF
)


@jit(nopython=True, cache=True)
def _initialize_tableau(A_ub, b_ub, A_eq, b_eq, tableau, basis):
    """
    Initialize the `tableau` and `basis` arrays in place for Phase 1.

    Suppose that the original linear program has the following form:

    maximize::

        c @ x

    subject to::

        A_ub @ x <= b_ub
        A_eq @ x == b_eq
               x >= 0

    Let s be a vector of slack variables converting the inequality
    constraint to an equality constraint so that the problem turns to be
    the standard form:

    maximize::

        c @ x

    subject to::

        A_ub @ x + s == b_ub
        A_eq @ x     == b_eq
        x, s         >= 0

    Then, let (z1, z2) be a vector of artificial variables for Phase 1.
    We solve the following LP:

    maximize::

        -(1 @ z1 + 1 @ z2)

    subject to::

        A_ub @ x + s + z1 == b_ub
        A_eq @ x + z2     == b_eq
        x, s, z1, z2      >= 0

    The tableau needs to be of shape (L+1, n+m+L+1), where L=m+k.

    Parameters
    ----------
    A_ub : ndarray(float, ndim=2)
        ndarray of shape (m, n).

    b_ub : ndarray(float, ndim=1)
        ndarray of shape (m,).

    A_eq : ndarray(float, ndim=2)
        ndarray of shape (k, n).

    b_eq : ndarray(float, ndim=1)
        ndarray of shape (k,).

    tableau : ndarray(float, ndim=2)
        Empty ndarray of shape (L+1, n+m+L+1) to store the tableau.
        Modified in place.

    basis : ndarray(int, ndim=1)
        Empty ndarray of shape (L,) to store the basic variables.
        Modified in place.

    Returns
    -------
    tableau : ndarray(float, ndim=2)
        View to `tableau`.

    basis : ndarray(int, ndim=1)
        View to `basis`.

    """
    m, k = A_ub.shape[0], A_eq.shape[0]
    L = m + k
    n = tableau.shape[1] - (m+L+1)

    for i in range(m):
        for j in range(n):
            tableau[i, j] = A_ub[i, j]
    for i in range(k):
        for j in range(n):
            tableau[m+i, j] = A_eq[i, j]

    tableau[:L, n:-1] = 0

    for i in range(m):
        tableau[i, -1] = b_ub[i]
        if tableau[i, -1] < 0:
            for j in range(n):
                tableau[i, j] *= -1
            tableau[i, n+i] = -1
            tableau[i, -1] *= -1
        else:
            tableau[i, n+i] = 1
        tableau[i, n+m+i] = 1
    for i in range(k):
        tableau[m+i, -1] = b_eq[i]
        if tableau[m+i, -1] < 0:
            for j in range(n):
                tableau[m+i, j] *= -1
            tableau[m+i, -1] *= -1
        tableau[m+i, n+m+m+i] = 1

    tableau[-1, :] = 0
    for i in range(L):
        for j in range(n+m):
            tableau[-1, j] += tableau[i, j]
        tableau[-1, -1] += tableau[i, -1]

    for i in range(L):
        basis[i] = n+m+i

    return tableau, basis


@jit(nopython=True, cache=True)
def _set_criterion_row(c, basis, tableau):
    """
    Modify the criterion row of the tableau for Phase 2.

    Parameters
    ----------
    c : ndarray(float, ndim=1)
        ndarray of shape (n,).

    basis : ndarray(int, ndim=1)
        ndarray of shape (L,) containing the basis obtained by Phase 1.

    tableau : ndarray(float, ndim=2)
        ndarray of shape (L+1, n+m+L+1) containing the tableau obtained
        by Phase 1. Modified in place.

    Returns
    -------
    tableau : ndarray(float, ndim=2)
        View to `tableau`.

    """
    n = c.shape[0]
    L = basis.shape[0]

    for j in range(n):
        tableau[-1, j] = c[j]
    tableau[-1, n:] = 0

    for i in range(L):
        multiplier = tableau[-1, basis[i]]
        for j in range(tableau.shape[1]):
            tableau[-1, j] -= tableau[i, j] * multiplier

    return tableau


@jit(nopython=True, cache=True)
def solve_tableau(tableau, basis, max_iter=10**6, skip_aux=True,
                  piv_options=PivOptions()):
    """
    Perform the simplex algorithm on a given tableau in canonical form.

    Used to solve a linear program in the following form:

    maximize::

        c @ x

    subject to::

        A_ub @ x <= b_ub
        A_eq @ x == b_eq
               x >= 0

    where A_ub is of shape (m, n) and A_eq is of shape (k, n). Thus,
    `tableau` is of shape (L+1, n+m+L+1), where L=m+k, and

    * `tableau[np.arange(L), :][:, basis]` must be an identity matrix,
      and
    * the elements of `tableau[:-1, -1]` must be nonnegative.

    Parameters
    ----------
    tableau : ndarray(float, ndim=2)
        ndarray of shape (L+1, n+m+L+1) containing the tableau. Modified
        in place.

    basis : ndarray(int, ndim=1)
        ndarray of shape (L,) containing the basic variables. Modified
        in place.

    max_iter : int, optional(default=10**6)
        Maximum number of pivoting steps.

    skip_aux : bool, optional(default=True)
        Whether to skip the coefficients of the auxiliary (or
        artificial) variables in pivot column selection.

    piv_options : PivOptions, optional
        PivOptions namedtuple to set the tolerance values.

    Returns
    -------
    success : bool
        True if the algorithm succeeded in finding an optimal solution.

    status : int
        An integer representing the exit status of the optimization.

    num_iter : int
        The number of iterations performed.

    """
    L = tableau.shape[0] - 1

    # Array to store row indices in lex_min_ratio_test
    argmins = np.empty(L, dtype=np.int_)

    success = False
    status = 1
    num_iter = 0

    while num_iter < max_iter:
        num_iter += 1

        pivcol_found, pivcol = _pivot_col(tableau, skip_aux, piv_options)

        if not pivcol_found:  # Optimal
            success = True
            status = 0
            break

        aux_start = tableau.shape[1] - L - 1
        pivrow_found, pivrow = _lex_min_ratio_test(
            tableau[:-1, :], pivcol, aux_start, argmins,
            piv_options.tol_piv, piv_options.tol_ratio_diff
        )

        if not pivrow_found:  # Unbounded
            success = False
            status = 3
            break

        _pivoting(tableau, pivcol, pivrow)
        basis[pivrow] = pivcol

    return success, status, num_iter


@jit(nopython=True, cache=True)
def _pivot_col(tableau, skip_aux, piv_options):
    """
    Choose the column containing the pivot element: the column
    containing the maximum positive element in the last row of the
    tableau.

    `skip_aux` should be True in phase 1, and False in phase 2.

    Parameters
    ----------
    tableau : ndarray(float, ndim=2)
        ndarray of shape (L+1, n+m+L+1) containing the tableau.

    skip_aux : bool
        Whether to skip the coefficients of the auxiliary (or
        artificial) variables in pivot column selection.

    piv_options : PivOptions, optional
        PivOptions namedtuple to set the tolerance values.

    Returns
    -------
    found : bool
        True iff there is a positive element in the last row of the
        tableau (and then pivoting should be conducted).

    pivcol : int
        The index of column containing the pivot element. (-1 if `found
        == False`.)

    """
    L = tableau.shape[0] - 1
    criterion_row_stop = tableau.shape[1] - 1
    if skip_aux:
        criterion_row_stop -= L

    found = False
    pivcol = -1
    coeff = piv_options.fea_tol
    for j in range(criterion_row_stop):
        if tableau[-1, j] > coeff:
            coeff = tableau[-1, j]
            pivcol = j
            found = True

    return found, pivcol


@jit(nopython=True, cache=True)
def get_solution(tableau, basis, x, lambd, b_signs):
    """
    Fetch the optimal solution and value from an optimal tableau.

    Parameters
    ----------
    tableau : ndarray(float, ndim=2)
        ndarray of shape (L+1, n+m+L+1) containing the optimal tableau,
        where L=m+k.

    basis : ndarray(int, ndim=1)
        ndarray of shape (L,) containing the optimal basis.

    x : ndarray(float, ndim=1)
        Empty ndarray of shape (n,) to store the primal solution.
        Modified in place.

    lambd : ndarray(float, ndim=1)
        Empty ndarray of shape (L,) to store the dual solution. Modified
        in place.

    b_signs : ndarray(bool, ndim=1)
        ndarray of shape (L,) whose i-th element is True iff the i-th
        element of the vector (b_ub, b_eq) is positive.

    Returns
    -------
    fun : float
        The optimal value.

    """
    n, L = x.size, lambd.size
    aux_start = tableau.shape[1] - L - 1

    x[:] = 0
    for i in range(L):
        if basis[i] < n:
            x[basis[i]] = tableau[i, -1]
    for j in range(L):
        lambd[j] = tableau[-1, aux_start+j]
        if lambd[j] != 0 and b_signs[j]:
            lambd[j] *= -1
    fun = tableau[-1, -1] * (-1)

    return fun



"""
Tests for game_theory/random.py

"""
import numpy as np
from numpy.testing import (
    assert_allclose, assert_raises, assert_, assert_array_equal
)
from quantecon.game_theory import (
    random_game, covariance_game, random_pure_actions, random_mixed_actions,
)


def test_random_game():
    nums_actions = (2, 3, 4)
    g = random_game(nums_actions)
    assert_(g.nums_actions == nums_actions)

    # Generate seed by np.random.SeedSequence().entropy
    seed = 227108210370342174739429861866005407311
    gs = [
        random_game(nums_actions, random_state=np.random.default_rng(seed))
        for i in range(2)
    ]
    assert_array_equal(*[g.payoff_profile_array for g in gs])


def test_covariance_game():
    nums_actions = (2, 3, 4)
    N = len(nums_actions)

    rho = 0.5
    g = covariance_game(nums_actions, rho=rho)
    assert_(g.nums_actions == nums_actions)

    seed = 289722416785475140936980467255496855908
    gs = [
        covariance_game(nums_actions, rho=rho,
                        random_state=np.random.default_rng(seed))
        for i in range(2)
    ]
    assert_array_equal(*[g.payoff_profile_array for g in gs])

    rho = 1
    g = covariance_game(nums_actions, rho=rho)
    for a in np.ndindex(*nums_actions):
        payoff_profile = g.payoff_profile_array[a]
        for i in range(N-1):
            assert_allclose(payoff_profile[i], payoff_profile[-1], atol=1e-7)

    rho = -1 / (N - 1)
    g = covariance_game(nums_actions, rho=rho)
    for a in np.ndindex(*nums_actions):
        assert_allclose(g.payoff_profile_array.sum(axis=-1),
                        np.zeros(nums_actions),
                        atol=1e-10)


def test_random_game_value_error():
    nums_actions = ()  # empty
    assert_raises(ValueError, random_game, nums_actions)


def test_covariance_game_value_error():
    nums_actions = ()  # empty
    assert_raises(ValueError, covariance_game, nums_actions, rho=0)

    nums_actions = (2,)  # length one
    assert_raises(ValueError, covariance_game, nums_actions, rho=0)

    nums_actions = (2, 3, 4)

    rho = 1.1  # > 1
    assert_raises(ValueError, covariance_game, nums_actions, rho)

    rho = -1  # < -1/(N-1)
    assert_raises(ValueError, covariance_game, nums_actions, rho)


def test_random_pure_actions():
    nums_actions = (2, 3, 4)
    N = len(nums_actions)
    seed = 1234
    for gen in [lambda x: x, np.random.default_rng]:
        action_profiles = [
            random_pure_actions(nums_actions, gen(seed)) for i in range(2)
        ]
        for i in range(N):
            assert_(action_profiles[0][i] < nums_actions[i])
        assert_(action_profiles[0] == action_profiles[1])


def test_random_mixed_actions():
    nums_actions = (2, 3, 4)
    seed = 1234
    action_profile = random_mixed_actions(nums_actions, seed)
    assert_(tuple([len(action) for action in action_profile]) == nums_actions)




"""
Tests for inequality.py

"""

import numpy as np
from numpy.testing import assert_allclose, assert_raises
from scipy.stats import linregress
from quantecon import lorenz_curve, gini_coefficient, \
     shorrocks_index, rank_size


def test_lorenz_curve():
    """
    Tests `lorenz` function, which calculates the lorenz curve

    An income distribution where everyone has almost the same wealth should
    be similar to a straight line

    An income distribution where one person has almost the wealth should
    be flat and then shoot straight up when it approaches one
    """
    n = 3000

    # Almost Equal distribution
    y = np.repeat(1, n) + np.random.normal(scale=0.0001, size=n)
    cum_people, cum_income = lorenz_curve(y)
    assert_allclose(cum_people, cum_income, rtol=1e-03)

    # Very uneven distribution
    y = np.repeat(0.001, n)
    y[4] = 100000
    pop_cum, income_cum = lorenz_curve(y)
    expected_income_cum = np.repeat(0., n + 1)
    expected_income_cum[-1] = 1.
    assert_allclose(expected_income_cum, income_cum, atol=1e-4)


def test_gini_coeff():
    """
    Tests how the function `gini_coefficient` calculates the Gini coefficient
    with the Pareto and the Weibull distribution.

    Analytically, we know that Pareto with parameter `a` has
    G = 1 / (2*a - 1)

    Likewise, for the Weibull distribution with parameter `a` we know that
    G = 1 - 2**(-1/a)

    """
    n = 10000

    # Tests Pareto: G = 1 / (2*a - 1)
    a = np.random.randint(2, 15)
    expected = 1 / (2 * a - 1)

    y = (np.random.pareto(a, size=n) + 1) * 2
    coeff = gini_coefficient(y)
    assert_allclose(expected, coeff, rtol=1e-01)

    # Tests Weibull: G = 1 - 2**(-1/a)
    a = np.random.randint(2, 15)
    expected = 1 - 2 ** (-1 / a)

    y = np.random.weibull(a, size=n)
    coeff = gini_coefficient(y)
    assert_allclose(expected, coeff, rtol=1e-01)


def test_shorrocks_index():
    """
    Test Shorrocks mobility index function against the example used in 'Wealth
    distribution and social mobility in the US: A quantitative approach'
    (Benhabib, Bisin, Luo, 2017).''
    https://www.econ.nyu.edu/user/bisina/RevisionAugust.pdf
    """

    # Construct the mobility matrix from Benhabib et al.
    P = [[0.222, 0.222, 0.215, 0.187, 0.081, 0.038, 0.029, 0.006],
         [0.221, 0.220, 0.215, 0.188, 0.082, 0.039, 0.029, 0.006],
         [0.207, 0.209, 0.210, 0.194, 0.090, 0.046, 0.036, 0.008],
         [0.198, 0.201, 0.207, 0.198, 0.095, 0.052, 0.040, 0.009],
         [0.175, 0.178, 0.197, 0.207, 0.110, 0.067, 0.054, 0.012],
         [0.182, 0.184, 0.200, 0.205, 0.106, 0.062, 0.050, 0.011],
         [0.123, 0.125, 0.166, 0.216, 0.141, 0.114, 0.094, 0.021],
         [0.084, 0.084, 0.142, 0.228, 0.170, 0.143, 0.121, 0.028]]

    expected = 0.98  # result from paper
    index = shorrocks_index(P)
    assert_allclose(expected, index, rtol=1e-2)


def test_rank_size():
    """
    Tests `rank_size` function, which generates rank-size data for
    a Pareto distribution.

    The rank-size plot for a sample drawn from a Pareto distribution
    should be a straight line.

    The length of the `rank_data` array should be within (c x 100)%
    of the size of the distribution.
    """

    rng = np.random.default_rng(15)
    sample_size = 10000
    c = 0.74

    # Tests Pareto; r_squared ~ 1
    pareto_draw = np.exp(rng.exponential(scale=1.0, size=sample_size))
    rank_data, size_data = rank_size(pareto_draw, c=c)

    assert len(rank_data) == len(size_data)
    assert_allclose(c*sample_size, len(rank_data), rtol=1e-3)

    _, _, r_value, _, _ = linregress(np.log(rank_data), np.log(size_data))
    r_sqval = r_value**2

    assert_allclose(r_sqval, 1, rtol=1e-3)

    # Tests Exponential; r_squared < 1
    z = rng.standard_normal(sample_size)

    exp_draw = np.exp(z)
    rank_data_exp, size_data_exp = rank_size(exp_draw, c=c)

    _, _, r_value_exp, _, _ = linregress(np.log(rank_data_exp),
                                         np.log(size_data_exp))
    r_sqval_exp = r_value_exp**2

    assert_raises(AssertionError, assert_allclose, r_sqval_exp, 1, rtol=1e-3)




print("import: 'quantecon'")
import quantecon

print("import: 'quantecon.tests'")
import quantecon.tests

print("import: 'quantecon.game_theory'")
import quantecon.game_theory

print("import: 'quantecon.markov'")
import quantecon.markov

print("import: 'quantecon.random'")
import quantecon.random

print("import: 'quantecon.util'")
import quantecon.util




"""
Tests for markov/core.py

Functions
---------
    mc_compute_stationary   [Status: Tested in test_markovchain_pmatrices]
    mc_sample_path          [Status: Tested in test_mc_sample_path]

"""
import numpy as np
from scipy import sparse
import itertools
from numpy.testing import (
    assert_allclose, assert_array_equal, assert_array_less, assert_raises,
    assert_, assert_equal
)

from quantecon.markov import (
    MarkovChain, mc_compute_stationary, mc_sample_path
)


def list_of_array_equal(s, t):
    """
    Compare two lists of ndarrays

    s, t: lists of numpy.ndarrays

    """
    assert_(len(s) == len(t))
    all(assert_array_equal(x, y) for x, y in zip(s, t))


# KMR Function
# Useful because it seems to have 1 unit eigvalue, but a second one that
# approaches unity.  Good test of accuracy.
def KMR_Markov_matrix_sequential(N, p, epsilon):
    """
    Generate the Markov matrix for the KMR model with *sequential* move

    N: number of players
    p: level of p-dominance for action 1
       = the value of p such that action 1 is the BR for (1-q, q) for any q > p,
         where q (1-q, resp.) is the prob that the opponent plays action 1 (0, resp.)
    epsilon: mutation probability

    References:
        KMRMarkovMatrixSequential is contributed from https://github.com/oyamad
    """
    P = np.zeros((N+1, N+1), dtype=float)
    P[0, 0], P[0, 1] = 1 - epsilon * (1/2), epsilon * (1/2)
    for n in range(1, N):
        P[n, n-1] = \
            (n/N) * (epsilon * (1/2) +
                     (1 - epsilon) * (((n-1)/(N-1) < p) + ((n-1)/(N-1) == p) * (1/2))
                     )
        P[n, n+1] = \
            ((N-n)/N) * (epsilon * (1/2) +
                         (1 - epsilon) * ((n/(N-1) > p) + (n/(N-1) == p) * (1/2))
                         )
        P[n, n] = 1 - P[n, n-1] - P[n, n+1]
    P[N, N-1], P[N, N] = epsilon * (1/2), 1 - epsilon * (1/2)
    return P


# Tests: methods of MarkovChain, mc_compute_stationary #

def test_markovchain_pmatrices():
    """
    Test the methods of MarkovChain, as well as mc_compute_stationary,
    with P matrix and known solutions
    """
    # Matrix with two recurrent classes [0, 1] and [3, 4, 5],
    # which have periods 2 and 3, respectively
    Q = np.zeros((6, 6))
    Q[0, 1], Q[1, 0] = 1, 1
    Q[2, [0, 3]] = 1/2
    Q[3, 4], Q[4, 5], Q[5, 3] = 1, 1, 1
    Q_stationary_dists = \
        np.array([[1/2, 1/2, 0, 0, 0, 0], [0, 0, 0, 1/3, 1/3, 1/3]])

    testset = [
        {'P': np.array([[0.4, 0.6], [0.2, 0.8]]),  # P matrix
         'stationary_dists': np.array([[0.25, 0.75]]),  # Known solution
         'comm_classes': [np.arange(2)],
         'rec_classes': [np.arange(2)],
         'is_irreducible': True,
         'period': 1,
         'is_aperiodic': True,
         'cyclic_classes': [np.arange(2)],
         },
        {'P': sparse.csr_matrix([[0.4, 0.6], [0.2, 0.8]]),
         'stationary_dists': np.array([[0.25, 0.75]]),
         'comm_classes': [np.arange(2)],
         'rec_classes': [np.arange(2)],
         'is_irreducible': True,
         'period': 1,
         'is_aperiodic': True,
         'cyclic_classes': [np.arange(2)],
         },
        {'P': np.array([[0, 1], [1, 0]]),
         'stationary_dists': np.array([[0.5, 0.5]]),
         'comm_classes': [np.arange(2)],
         'rec_classes': [np.arange(2)],
         'is_irreducible': True,
         'period': 2,
         'is_aperiodic': False,
         'cyclic_classes': [np.array([0]), np.array([1])],
         },
        {'P': np.eye(2),
         'stationary_dists': np.array([[1, 0], [0, 1]]),
         'comm_classes': [np.array([0]), np.array([1])],
         'rec_classes': [np.array([0]), np.array([1])],
         'is_irreducible': False,
         'period': 1,
         'is_aperiodic': True,
         },
        # Reducible mc with a unique recurrent class,
        # where n-1 is a transient state
        {'P': np.array([[1, 0], [1, 0]]),
         'stationary_dists': np.array([[1, 0]]),
         'comm_classes': [np.array([0]), np.array([1])],
         'rec_classes': [np.array([0])],
         'is_irreducible': False,
         'period': 1,
         'is_aperiodic': True,
         },
        {'P': Q,
         'stationary_dists': Q_stationary_dists,
         'comm_classes': [np.array([0, 1]), np.array([2]), np.array([3, 4, 5])],
         'rec_classes': [np.array([0, 1]), np.array([3, 4, 5])],
         'is_irreducible': False,
         'period': 6,
         'is_aperiodic': False,
         },
        {'P': sparse.csr_matrix(Q),
         'stationary_dists': Q_stationary_dists,
         'comm_classes': [np.array([0, 1]), np.array([2]), np.array([3, 4, 5])],
         'rec_classes': [np.array([0, 1]), np.array([3, 4, 5])],
         'is_irreducible': False,
         'period': 6,
         'is_aperiodic': False,
         }
    ]

    # Loop Through TestSet #
    for test_dict in testset:
        mc = MarkovChain(test_dict['P'])
        computed = mc.stationary_distributions
        assert_allclose(computed, test_dict['stationary_dists'])

        assert(mc.num_communication_classes == len(test_dict['comm_classes']))
        assert(mc.is_irreducible == test_dict['is_irreducible'])
        assert(mc.num_recurrent_classes == len(test_dict['rec_classes']))
        list_of_array_equal(
            sorted(mc.communication_classes, key=lambda x: x[0]),
            sorted(test_dict['comm_classes'], key=lambda x: x[0])
        )
        list_of_array_equal(
            sorted(mc.recurrent_classes, key=lambda x: x[0]),
            sorted(test_dict['rec_classes'], key=lambda x: x[0])
        )
        assert(mc.period == test_dict['period'])
        assert(mc.is_aperiodic == test_dict['is_aperiodic'])
        try:
            list_of_array_equal(
                sorted(mc.cyclic_classes, key=lambda x: x[0]),
                sorted(test_dict['cyclic_classes'], key=lambda x: x[0])
            )
        except NotImplementedError:
            assert(mc.is_irreducible is False)

        # Test of mc_compute_stationary
        computed = mc_compute_stationary(test_dict['P'])
        assert_allclose(computed, test_dict['stationary_dists'])


# Basic Class Structure with Setup #
####################################

class Test_markovchain_stationary_distributions_KMRMarkovMatrix2():
    """
    Test Suite for MarkovChain.stationary_distributions using KMR Markov
    Matrix [suitable for nose]
    """

    # Starting Values #

    N = 27
    epsilon = 1e-2
    p = 1/3
    TOL = 1e-2

    def setup_method(self):
        """ Setup a KMRMarkovMatrix and Compute Stationary Values """
        self.P = KMR_Markov_matrix_sequential(self.N, self.p, self.epsilon)
        self.mc = MarkovChain(self.P)
        self.stationary = self.mc.stationary_distributions
        stat_shape = self.stationary.shape

        if len(stat_shape) == 1:
            self.n_stat_dists = 1
        else:
            self.n_stat_dists = stat_shape[0]

    def test_markov_matrix(self):
        "Check that each row of matrix sums to 1"
        mc = self.mc
        assert_allclose(np.sum(mc.P, axis=1), np.ones(mc.n))

    def test_sum_one(self):
        "Check that each stationary distribution sums to 1"
        stationary_distributions = self.stationary
        assert_allclose(np.sum(stationary_distributions, axis=1),
                        np.ones(self.n_stat_dists))

    def test_nonnegative(self):
        "Check that the stationary distributions are non-negative"
        stationary_distributions = self.stationary
        assert(np.all(stationary_distributions > -1e-16))

    def test_left_eigen_vec(self):
        "Check that vP = v for all stationary distributions"
        mc = self.mc
        stationary_distributions = self.stationary

        if self.n_stat_dists == 1:
            assert_allclose(np.dot(stationary_distributions, mc.P),
                            stationary_distributions, atol=self.TOL)
        else:
            for i in range(self.n_stat_dists):
                curr_v = stationary_distributions[i, :]
                assert_allclose(np.dot(curr_v, mc.P), curr_v, atol=self.TOL)


def test_simulate_shape():
    P = [[0.4, 0.6], [0.2, 0.8]]
    mcs = [MarkovChain(P), MarkovChain(sparse.csr_matrix(P))]

    for mc in mcs:
        (ts_length, init, num_reps) = (10, None, None)
        assert_array_equal(mc.simulate(ts_length, init, num_reps).shape,
                           (ts_length,))

        (ts_length, init, num_reps) = (10, [0, 1], None)
        assert_array_equal(mc.simulate(ts_length, init, num_reps).shape,
                           (len(init), ts_length))

        (ts_length, init, num_reps) = (10, [0, 1], 3)
        assert_array_equal(mc.simulate(ts_length, init, num_reps).shape,
                           (len(init)*num_reps, ts_length))

        for (ts_length, init, num_reps) in [(10, None, 3), (10, None, 1)]:
            assert_array_equal(mc.simulate(ts_length, init, num_reps).shape,
                               (num_reps, ts_length))


def test_simulate_init_array_num_reps():
    P = [[0.4, 0.6], [0.2, 0.8]]
    mc = MarkovChain(P)

    ts_length = 10
    init = [0, 1]
    num_reps = 3

    X = mc.simulate(ts_length, init, num_reps)
    assert_array_equal(X[:, 0], init*num_reps)


def test_simulate_init_type():
    P = [[0.4, 0.6], [0.2, 0.8]]
    mc = MarkovChain(P)

    seed = 0
    ts_length = 3
    init = 0  # int
    X = mc.simulate(ts_length, init=init, random_state=seed)

    inits_np_int = [t(init) for t in [np.int32, np.int64]]
    for init in inits_np_int:
        X_np_int = mc.simulate(ts_length, init=init, random_state=seed)
        assert_array_equal(X_np_int, X)


def test_simulate_dense_vs_sparse():
    n = 5
    a = 1/3
    b = (1 - a)/2
    P = np.zeros((n, n))
    for i in range(n):
        P[i, (i-1) % n], P[i, i], P[i, (i+1) % n] = b, a, b
    mcs = [MarkovChain(P), MarkovChain(sparse.csr_matrix(P))]

    ts_length = 10
    inits = (None, 0, [0, 1])
    num_repss = (None, 2)

    random_state = 0
    for init, num_reps in itertools.product(inits, num_repss):
        assert_array_equal(*(mc.simulate(ts_length, init, num_reps,
                                         random_state=random_state)
                             for mc in mcs))


def test_simulate_ergodicity():
    P = [[0.4, 0.6], [0.2, 0.8]]
    stationary_dist = [0.25, 0.75]
    init = 0
    mc = MarkovChain(P)

    seed = 4433
    ts_length = 100
    num_reps = 300
    tol = 0.1

    x = mc.simulate(ts_length, init=init, num_reps=num_reps, random_state=seed)
    frequency_1 = x[:, -1].mean()
    assert_(np.abs(frequency_1 - stationary_dist[1]) < tol)


def test_simulate_for_matrices_with_C_F_orders():
    """
    Test MarkovChasin.simulate for matrices with C- and F-orders
    See the issue and fix on Numba:
    github.com/numba/numba/issues/1103
    github.com/numba/numba/issues/1104
    """
    P_C = np.array([[0.5, 0.5], [0, 1]], order='C')
    P_F = np.array([[0.5, 0.5], [0, 1]], order='F')
    init = 1
    ts_length = 10
    sample_path = np.ones(ts_length, dtype=int)

    computed_C_and_F = \
        MarkovChain(np.array([[1.]])).simulate(ts_length, init=0)
    assert_array_equal(computed_C_and_F, np.zeros(ts_length, dtype=int))

    computed_C = MarkovChain(P_C).simulate(ts_length, init)
    computed_F = MarkovChain(P_F).simulate(ts_length, init=init)
    assert_array_equal(computed_C, sample_path)
    assert_array_equal(computed_F, sample_path)


def test_simulate_issue591():
    """
    Test MarkovChasin.simulate for P with dtype=np.float32
    https://github.com/QuantEcon/QuantEcon.py/issues/591
    """
    num_states = 5
    transition_states = 4

    transition_seed = 2
    random_state = np.random.RandomState(transition_seed)
    transitions = random_state.uniform(0., 1., transition_states)
    transitions /= np.sum(transitions)
    P = np.zeros((num_states, num_states), dtype=np.float32)
    P[0, :transition_states] = transitions
    P[1:, 0] = 1.
    mc = MarkovChain(P=P)

    simulate_seed = 22220
    ts_length = 10000
    seq = mc.simulate(
        ts_length=ts_length, init=0, num_reps=1, random_state=simulate_seed
    )
    max_state_in_seq = np.max(seq)

    assert_array_less(max_state_in_seq, num_states)


def test_mc_sample_path():
    P = [[0.4, 0.6], [0.2, 0.8]]
    Ps = [P, sparse.csr_matrix(P)]

    init = 0
    sample_size = 10

    seed = 42

    for P in Ps:
        # init: integer
        expected = [0, 0, 1, 1, 1, 0, 0, 0, 1, 1]
        computed = mc_sample_path(
            P, init=init, sample_size=sample_size, random_state=seed
        )
        assert_array_equal(computed, expected)

        # init: distribution
        distribution = (0.5, 0.5)
        expected = [0, 1, 1, 1, 0, 0, 0, 1, 1, 1]
        computed = mc_sample_path(
            P, init=distribution, sample_size=sample_size, random_state=seed
        )
        assert_array_equal(computed, expected)


def test_mc_sample_path_lln():
    P = [[0.4, 0.6], [0.2, 0.8]]
    stationary_dist = [0.25, 0.75]
    init = 0

    seed = 4433
    sample_size = 10**4
    tol = 0.02

    frequency_1 = mc_sample_path(P, init=init, sample_size=sample_size,
                                 random_state=seed).mean()
    assert_(np.abs(frequency_1 - stationary_dist[1]) < tol)


class TestMCStateValues:
    def setup_method(self):
        state_values = [[0, 1], [2, 3], [4, 5]]  # Pass python list
        self.state_values = np.array(state_values)

        self.mc_reducible_dict = {
            'mc': MarkovChain([[1, 0, 0], [1, 0, 0], [0, 0, 1]],
                              state_values=state_values),
            'coms': [[0], [1], [2]],
            'recs': [[0], [2]],
            'period': 1
        }

        self.mc_periodic_dict = {
            'mc': MarkovChain([[0, 1, 0], [0, 0, 1], [1, 0, 0]],
                              state_values=state_values),
            'coms': [[0, 1, 2]],
            'recs': [[0, 1, 2]],
            'cycs': [[0], [1], [2]],
            'period': 3
        }

    def test_com_rec_classes(self):
        for mc_dict in [self.mc_reducible_dict, self.mc_periodic_dict]:
            mc = mc_dict['mc']
            coms = mc_dict['coms']
            recs = mc_dict['recs']
            properties = ['communication_classes',
                          'recurrent_classes']
            suffix = '_indices'
            for prop0, classes_ind in zip(properties, [coms, recs]):
                for return_indices in [True, False]:
                    if return_indices:
                        classes = classes_ind
                        prop = prop0 + suffix
                        key = lambda x: x[0]
                    else:
                        classes = [self.state_values[i] for i in classes_ind]
                        prop = prop0
                        key = lambda x: x[0, 0]
                    list_of_array_equal(
                        sorted(getattr(mc, prop), key=key),
                        sorted(classes, key=key)
                    )

    def test_cyc_classes(self):
        mc = self.mc_periodic_dict['mc']
        cycs = self.mc_periodic_dict['cycs']
        properties = ['cyclic_classes']
        suffix = '_indices'
        for prop0, classes_ind in zip(properties, [cycs]):
            for return_indices in [True, False]:
                if return_indices:
                    classes = classes_ind
                    prop = prop0 + suffix
                    key = lambda x: x[0]
                else:
                    classes = [self.state_values[i] for i in classes_ind]
                    prop = prop0
                    key = lambda x: x[0, 0]
                list_of_array_equal(
                    sorted(getattr(mc, prop), key=key),
                    sorted(classes, key=key)
                )

    def test_period(self):
        for mc_dict in [self.mc_reducible_dict, self.mc_periodic_dict]:
            mc = mc_dict['mc']
            period = mc_dict['period']
            assert_equal(mc.period, period)

    def test_simulate(self):
        # Deterministic mc
        mc = self.mc_periodic_dict['mc']
        ts_length = 6

        methods = ['simulate_indices', 'simulate']

        init_idx = 0
        inits = [init_idx, self.state_values[init_idx]]
        path = np.arange(init_idx, init_idx+ts_length)%mc.n
        paths = [path, self.state_values[path]]
        for method, init, X_expected in zip(methods, inits, paths):
            X = getattr(mc, method)(ts_length, init)
            assert_array_equal(X, X_expected)

        init_idx = [1, 2]
        inits = [init_idx, self.state_values[init_idx]]
        path = np.array(
            [np.arange(i, i+ts_length)%mc.n for i in init_idx]
        )
        paths = [path, self.state_values[path]]
        for method, init, X_expected in zip(methods, inits, paths):
            X = getattr(mc, method)(ts_length, init)
            assert_array_equal(X, X_expected)

        inits = [None, None]
        seed = 1234  # init will be 2
        init_idx = 2
        path = np.arange(init_idx, init_idx+ts_length)%mc.n
        paths = [path, self.state_values[path]]
        for method, init, X_expected in zip(methods, inits, paths):
            X = getattr(mc, method)(ts_length, init, random_state=seed)
            assert_array_equal(X, X_expected)


def test_mc_stationary_distributions_state_values():
    P = [[0.4, 0.6, 0], [0.2, 0.8, 0], [0, 0, 1]]
    state_values = ['a', 'b', 'c']
    mc = MarkovChain(P, state_values=state_values)
    stationary_dists_expected = [[0.25, 0.75, 0], [0, 0, 1]]
    stationary_dists = mc.stationary_distributions
    assert_allclose(stationary_dists, stationary_dists_expected)


def test_get_index():
    P = [[0.4, 0.6], [0.2, 0.8]]
    mc = MarkovChain(P)

    assert_(mc.get_index(0) == 0)
    assert_(mc.get_index(1) == 1)
    assert_raises(ValueError, mc.get_index, 2)
    assert_array_equal(mc.get_index([1, 0]), [1, 0])
    assert_raises(ValueError, mc.get_index, [[1]])

    mc.state_values = [1, 2]
    assert_(mc.get_index(1) == 0)
    assert_(mc.get_index(2) == 1)
    assert_raises(ValueError, mc.get_index, 0)
    assert_array_equal(mc.get_index([2, 1]), [1, 0])
    assert_raises(ValueError, mc.get_index, [[1]])

    mc.state_values = [[1, 2], [3, 4]]
    assert_(mc.get_index([1, 2]) == 0)
    assert_raises(ValueError, mc.get_index, 1)
    assert_array_equal(mc.get_index([[3, 4], [1, 2]]), [1, 0])


def test_raises_value_error_non_2dim():
    """Test with non 2dim input"""
    assert_raises(ValueError, MarkovChain, np.array([0.4, 0.6]))


def test_raises_value_error_non_sym():
    """Test with non symmetric input"""
    P = np.array([[0.4, 0.6]])
    assert_raises(ValueError, MarkovChain, P)
    assert_raises(ValueError, MarkovChain, sparse.csr_matrix(P))


def test_raises_value_error_non_nonnegative():
    """Test with non nonnegative input"""
    P = np.array([[0.4, 0.6], [-0.2, 1.2]])
    assert_raises(ValueError, MarkovChain, P)
    assert_raises(ValueError, MarkovChain, sparse.csr_matrix(P))


def test_raises_value_error_non_sum_one():
    """Test with input such that some of the rows does not sum to one"""
    P = np.array([[0.4, 0.6], [0.2, 0.9]])
    assert_raises(ValueError, MarkovChain, P)
    assert_raises(ValueError, MarkovChain, sparse.csr_matrix(P))


def test_raises_value_error_simulate_init_out_of_range():
    P = [[0.4, 0.6], [0.2, 0.8]]
    mc = MarkovChain(P)

    n = mc.n
    ts_length = 3
    assert_raises(ValueError, mc.simulate, ts_length, init=n)
    assert_raises(ValueError, mc.simulate, ts_length, init=-(n+1))
    assert_raises(ValueError, mc.simulate, ts_length, init=[0, n])
    assert_raises(ValueError, mc.simulate, ts_length, init=[0, -(n+1)])


def test_raises_non_homogeneous_state_values():
    P = [[0.4, 0.6], [0.2, 0.8]]
    state_values = [(0, 1), 2]
    assert_raises(ValueError, MarkovChain, P, state_values=state_values)



"""
Generate random NormalFormGame instances.

"""

import numpy as np

from .normal_form_game import Player, NormalFormGame
from ..util import check_random_state, rng_integers
from ..random.utilities import _probvec_cpu


def random_game(nums_actions, random_state=None):
    """
    Return a random NormalFormGame instance where the payoffs are drawn
    independently from the uniform distribution on [0, 1).

    Parameters
    ----------
    nums_actions : tuple(int)
        Tuple of the numbers of actions, one for each player.

    random_state : int or np.random.RandomState/Generator, optional
        Random seed (integer) or np.random.RandomState or Generator
        instance to set the initial state of the random number generator
        for reproducibility. If None, a randomly initialized RandomState
        is used.

    Returns
    -------
    g : NormalFormGame

    Examples
    --------
    >>> rng = np.random.default_rng(12345)
    >>> g = random_game((3, 2), random_state=rng)
    >>> print(g)
    2-player NormalFormGame with payoff profile array:
    [[[0.22733602, 0.59830875],  [0.31675834, 0.94180287]],
     [[0.79736546, 0.18673419],  [0.67625467, 0.24824571]],
     [[0.39110955, 0.67275604],  [0.33281393, 0.94888115]]]

    """
    N = len(nums_actions)
    if N == 0:
        raise ValueError('nums_actions must be non-empty')

    random_state = check_random_state(random_state)
    players = [
        Player(random_state.random(nums_actions[i:]+nums_actions[:i]))
        for i in range(N)
    ]
    g = NormalFormGame(players)
    return g


def covariance_game(nums_actions, rho, random_state=None):
    """
    Return a random NormalFormGame instance where the payoff profiles
    are drawn independently from the standard multi-normal with the
    covariance of any pair of payoffs equal to `rho`, as studied in
    [1]_.

    Parameters
    ----------
    nums_actions : tuple(int)
        Tuple of the numbers of actions, one for each player.

    rho : scalar(float)
        Covariance of a pair of payoff values. Must be in [-1/(N-1), 1],
        where N is the number of players.

    random_state : int or np.random.RandomState/Generator, optional
        Random seed (integer) or np.random.RandomState or Generator
        instance to set the initial state of the random number generator
        for reproducibility. If None, a randomly initialized RandomState
        is used.

    Returns
    -------
    g : NormalFormGame

    Examples
    --------
    >>> rng = np.random.default_rng(12345)
    >>> g = covariance_game((3, 2), rho=-0.7, random_state=rng)
    >>> print(g)
    2-player NormalFormGame with payoff profile array:
    [[[ 1.80214175, -0.8232619 ],  [ 0.7023331 , -0.90308782]],
     [[-0.2174803 , -0.35640649],  [ 1.51235766, -1.00972746]],
     [[-1.08921974, -0.42346148],  [-1.78910753,  2.53930201]]]

    References
    ----------
    .. [1] Y. Rinott and M. Scarsini, "On the Number of Pure Strategy
       Nash Equilibria in Random Games," Games and Economic Behavior
       (2000), 274-293.

    """
    N = len(nums_actions)
    if N <= 1:
        raise ValueError('length of nums_actions must be at least 2')
    if not (-1 / (N - 1) <= rho <= 1):
        lb = '-1' if N == 2 else '-1/{0}'.format(N-1)
        raise ValueError('rho must be in [{0}, 1]'.format(lb))

    mean = np.zeros(N)
    cov = np.empty((N, N))
    cov.fill(rho)
    cov[range(N), range(N)] = 1

    random_state = check_random_state(random_state)
    payoff_profile_array = \
        random_state.multivariate_normal(mean, cov, nums_actions)
    g = NormalFormGame(payoff_profile_array)
    return g


def random_pure_actions(nums_actions, random_state=None):
    """
    Return a tuple of random pure actions (integers).

    Parameters
    ----------
    nums_actions : tuple(int)
        Tuple of the numbers of actions, one for each player.

    random_state : int or np.random.RandomState/Generator, optional
        Random seed (integer) or np.random.RandomState or Generator
        instance to set the initial state of the random number generator
        for reproducibility. If None, a randomly initialized RandomState
        is used.

    Returns
    -------
    action_profile : Tuple(int)
        Tuple of actions, one for each player.

    """
    random_state = check_random_state(random_state)
    action_profile = tuple(
        [rng_integers(random_state, num_actions) for num_actions in nums_actions]
    )
    return action_profile


def _random_mixed_actions(out, random_state):
    """
    Fill the tuple `out` of empty ndarrays with random mixed actions.

    Parameters
    ----------
    out : tuple(ndarray(float, ndim=1))
        Tuple of 1d ndarrays, to be modified in place.

    random_state : np.random.RandomState or np.random.Generator

    Return
    ------
    out : tuple(ndarray(float, ndim=1))

    """
    for x in out:
        n = x.shape[0]
        if n == 1:
            x[0] = 1
        else:
            r = random_state.random(size=n-1)
            _probvec_cpu(r, x)
    return out


def random_mixed_actions(nums_actions, random_state=None):
    """
    Return a tuple of random mixed actions (vectors of floats).

    Parameters
    ----------
    nums_actions : tuple(int)
        Tuple of the numbers of actions, one for each player.

    random_state : int or np.random.RandomState/Generator, optional
        Random seed (integer) or np.random.RandomState or Generator
        instance to set the initial state of the random number generator
        for reproducibility. If None, a randomly initialized RandomState
        is used.

    Returns
    -------
    action_profile : tuple(ndarray(float, ndim=1))
        Tuple of mixed_actions, one for each player.

    """
    random_state = check_random_state(random_state)
    action_profile = \
        tuple(np.empty(num_actions) for num_actions in nums_actions)
    _random_mixed_actions(action_profile, random_state)
    return action_profile



"""
Utility routines for the game_theory submodule

"""
import numbers
import numpy as np


class NashResult(dict):
    """
    Contain the information about the result of Nash equilibrium
    computation.

    Attributes
    ----------
    NE : tuple(ndarray(float, ndim=1))
        Computed Nash equilibrium.

    converged : bool
        Whether the routine has converged.

    num_iter : int
        Number of iterations.

    max_iter : int
        Maximum number of iterations.

    init : scalar or array_like
        Initial condition used.

    Notes
    -----
    This is sourced from `sicpy.optimize.OptimizeResult`.

    There may be additional attributes not listed above depending of the
    routine.

    """
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, list(self.keys()))) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in sorted(self.items())])
        else:
            return self.__class__.__name__ + "()"

    def __dir__(self):
        return self.keys()


# _copy_action_to, _copy_action_profile_to

def _copy_action_to(dst, src):
    """
    Copy the pure action (int) or mixed action (array_like) in `src` to
    the empty ndarray `dst`.

    Parameters
    ----------
    dst : ndarray(float, ndim=1)

    src : scalar(int) or array_like(float, ndim=1)

    """
    if isinstance(src, numbers.Integral):  # pure action
        dst[:] = 0
        dst[src] = 1
    else:  # mixed action
        np.copyto(dst, src)


def _copy_action_profile_to(dst, src):
    """
    Copy the pure actions (int) or mixed actions (array_like) in the
    N-array_like `src` to the empty ndarrays in the N-array_like `dst`.

    Parameters
    ----------
    dst : array_like(ndarray(float, ndim=1))

    src : array_like(int or array_like(float, ndim=1))

    """
    N = len(dst)
    for i in range(N):
        _copy_action_to(dst[i], src[i])



"""
Filename: test_localint.py

Tests for localint.py

"""
import numpy as np
from numpy.testing import assert_array_equal, assert_equal

from quantecon.game_theory import LocalInteraction


class TestLocalInteraction:
    '''Test the methods of LocalInteraction'''

    def setup_method(self):
        '''Setup a LocalInteraction instance'''
        payoff_matrix = np.asarray([[4, 0], [2, 3]])
        adj_matrix = np.asarray([[0, 1, 3],
                                 [2, 0, 1],
                                 [3, 2, 0]])
        self.li = LocalInteraction(payoff_matrix, adj_matrix)

    def test_play(self):
        init_actions = (0, 0, 1)
        x = (1, 0, 0)
        assert_equal(self.li.play(actions=init_actions), x)

    def test_time_series_simultaneous_revision(self):
        init_actions = (0, 0, 1)
        x = [[0, 0, 1],
             [1, 0, 0],
             [0, 1, 1]]
        assert_array_equal(self.li.time_series(ts_length=3,
                                               actions=init_actions), x)

    def test_time_series_asynchronous_revision(self):
        seed = 21388225457580135037104913043871211454
        init_actions = (0, 0, 1)
        x = [self.li.time_series(ts_length=3,
                                 revision='asynchronous',
                                 actions=init_actions,
                                 random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_equal(x[0], x[1])

    def test_time_series_asynchronous_revision_with_player_index(self):
        init_actions = (0, 0, 1)
        player_ind_seq = [0, 1, 2]
        x = [[0, 0, 1],
             [1, 0, 1],
             [1, 1, 1]]
        assert_array_equal(self.li.time_series(ts_length=3,
                                               revision='asynchronous',
                                               actions=init_actions,
                                               player_ind_seq=player_ind_seq),
                           x)



"""
Tests for lae.py

TODO: write (economically) meaningful tests for this module

"""
import numpy as np
from numpy.testing import assert_
from scipy.stats import lognorm
from quantecon import LAE

# copied from the lae lecture
s = 0.2
delta = 0.1
a_sigma = 0.4       # A = exp(B) where B ~ N(0, a_sigma)
alpha = 0.4         # We set f(k) = k**alpha
phi = lognorm(a_sigma)


def p(x, y):
    d = s * x**alpha
    return phi.pdf((y - (1 - delta) * x) / d) / d

# other data
n_a, n_b, n_y = 50, (5, 5), 20
a = np.random.rand(n_a) + 0.01
b = np.random.rand(*n_b) + 0.01

y = np.linspace(0, 10, 20)

lae_a = LAE(p, a)
lae_b = LAE(p, b)


def test_x_flattened():
    "lae: is x flattened and reshaped"
    # should have a trailing singleton dimension
    assert_(lae_b.X.shape[-1] == 1)
    assert_(lae_a.X.shape[-1] == 1)


def test_x_2d():
    "lae: is x 2d"
    assert_(lae_a.X.ndim == 2)
    assert_(lae_b.X.ndim == 2)


def test_call_shapes():
    "lae: shape of call to lae"
    assert_(lae_a(y).shape == (n_y,))
    assert_(lae_b(y).shape == (n_y,))



"""
Tests for normal_form_game.py
"""
import numpy as np
from numpy.testing import assert_array_equal, assert_, assert_raises

from quantecon.game_theory import (
    Player, NormalFormGame, pure2mixed, best_response_2p
)


# Player #

LP_METHODS = [None, 'highs']


class TestPlayer_1opponent:
    """Test the methods of Player with one opponent player"""

    def setup_method(self):
        """Setup a Player instance"""
        coordination_game_matrix = [[4, 0], [3, 2]]
        self.player = Player(coordination_game_matrix)

    def test_delete_action(self):
        N = self.player.num_opponents + 1
        action_to_delete = 0
        actions_to_remain = \
            np.setdiff1d(np.arange(self.player.num_actions), action_to_delete)
        for i in range(N):
            player_new = self.player.delete_action(action_to_delete, i)
            assert_array_equal(
                player_new.payoff_array,
                self.player.payoff_array.take(actions_to_remain, axis=i)
            )

    def test_best_response_against_pure(self):
        assert_(self.player.best_response(1) == 1)

    def test_best_response_against_mixed(self):
        assert_(self.player.best_response([1/2, 1/2]) == 1)

    def test_best_response_list_when_tie(self):
        """best_response with tie_breaking=False"""
        assert_array_equal(
            sorted(self.player.best_response([2/3, 1/3], tie_breaking=False)),
            sorted([0, 1])
        )

    def test_best_response_with_random_tie_breaking(self):
        """best_response with tie_breaking='random'"""
        assert_(self.player.best_response([2/3, 1/3], tie_breaking='random')
                in [0, 1])

        seed = 1234
        brs = [
            self.player.best_response([2/3, 1/3], tie_breaking='random',
                                      random_state=seed)
            for i in range(2)
        ]
        assert_(brs[0] == brs[1])

        # Generate seed by np.random.SeedSequence().entropy
        seed = 189001345436880673361166627406341705095
        brs = [
            self.player.best_response([2/3, 1/3], tie_breaking='random',
                                      random_state=np.random.default_rng(seed))
            for i in range(2)
        ]
        assert_(brs[0] == brs[1])

    def test_best_response_with_smallest_tie_breaking(self):
        """best_response with tie_breaking='smallest' (default)"""
        assert_(self.player.best_response([2/3, 1/3]) == 0)

    def test_best_response_with_payoff_perturbation(self):
        """best_response with payoff_perturbation"""
        assert_(self.player.best_response([2/3, 1/3],
                payoff_perturbation=[0, 0.1]) == 1)
        assert_(self.player.best_response([2, 1],  # int
                payoff_perturbation=[0, 0.1]) == 1)

    def test_is_best_response_against_pure(self):
        assert_(self.player.is_best_response(0, 0))

    def test_is_best_response_against_mixed(self):
        assert_(self.player.is_best_response([1/2, 1/2], [2/3, 1/3]))

    def test_is_dominated(self):
        for action in range(self.player.num_actions):
            for method in LP_METHODS:
                assert_(not self.player.is_dominated(action, method=method))

    def test_dominated_actions(self):
        for method in LP_METHODS:
            assert_(self.player.dominated_actions(method=method) == [])


class TestPlayer_2opponents:
    """Test the methods of Player with two opponent players"""

    def setup_method(self):
        """Setup a Player instance"""
        payoffs_2opponents = [[[3, 6],
                               [4, 2]],
                              [[1, 0],
                               [5, 7]]]
        self.player = Player(payoffs_2opponents)

    def test_delete_action(self):
        N = self.player.num_opponents + 1
        action_to_delete = 0
        actions_to_remain = \
            np.setdiff1d(np.arange(self.player.num_actions), action_to_delete)
        for i in range(N):
            player_new = self.player.delete_action(action_to_delete, i)
            assert_array_equal(
                player_new.payoff_array,
                self.player.payoff_array.take(actions_to_remain, axis=i)
            )

    def test_payoff_vector_against_pure(self):
        assert_array_equal(self.player.payoff_vector((0, 1)), [6, 0])

    def test_is_best_response_against_pure(self):
        assert_(not self.player.is_best_response(0, (1, 0)))

    def test_best_response_against_pure(self):
        assert_(self.player.best_response((1, 1)) == 1)

    def test_best_response_list_when_tie(self):
        """
        best_response against a mixed action profile with
        tie_breaking=False
        """
        assert_array_equal(
            sorted(self.player.best_response(([3/7, 4/7], [1/2, 1/2]),
                                             tie_breaking=False)),
            sorted([0, 1])
        )

    def test_is_dominated(self):
        for action in range(self.player.num_actions):
            for method in LP_METHODS:
                assert_(not self.player.is_dominated(action, method=method))

    def test_dominated_actions(self):
        for method in LP_METHODS:
            assert_(self.player.dominated_actions(method=method) == [])


def test_random_choice():
    n, m = 5, 4
    payoff_matrix = np.zeros((n, m))
    player = Player(payoff_matrix)

    assert_(player.random_choice([0]) == 0)

    actions = list(range(player.num_actions))
    assert_(player.random_choice() in actions)


def test_player_corner_cases():
    n, m = 3, 4
    player = Player(np.zeros((n, m)))
    for action in range(n):
        assert_(player.is_best_response(action, [1/m]*m))
        for method in LP_METHODS:
            assert_(not player.is_dominated(action, method=method))

    e = 1e-8 * 2
    player = Player([[-e, -e], [1, -1], [-1, 1]])
    action = 0
    assert_(player.is_best_response(action, [1/2, 1/2], tol=e))
    assert_(not player.is_best_response(action, [1/2, 1/2], tol=e/2))
    for method in LP_METHODS:
        assert_(not player.is_dominated(action, tol=2*e, method=method))
        assert_(player.dominated_actions(tol=2*e, method=method) == [])

        assert_(player.is_dominated(action, tol=e/2, method=method))
        assert_(player.dominated_actions(tol=e/2, method=method) == [action])


# NormalFormGame #

class TestNormalFormGame_Sym2p:
    """Test the methods of NormalFormGame with symmetric two players"""

    def setup_method(self):
        """Setup a NormalFormGame instance"""
        coordination_game_matrix = [[4, 0], [3, 2]]
        self.g = NormalFormGame(coordination_game_matrix)

    def test_getitem(self):
        assert_array_equal(self.g[0, 1], [0, 3])

    def test_is_nash_pure(self):
        assert_(self.g.is_nash((0, 0)))

    def test_is_nash_mixed(self):
        assert_(self.g.is_nash(([2/3, 1/3], [2/3, 1/3])))


class TestNormalFormGame_Asym2p:
    """Test the methods of NormalFormGame with asymmetric two players"""

    def setup_method(self):
        """Setup a NormalFormGame instance"""
        self.BoS_bimatrix = np.array([[(3, 2), (1, 1)],
                                      [(0, 0), (2, 3)]])
        self.g = NormalFormGame(self.BoS_bimatrix)

    def test_getitem(self):
        action_profile = (1, 0)
        assert_array_equal(self.g[action_profile],
                           self.BoS_bimatrix[action_profile])

    def test_delete_action(self):
        action_to_delete = 0
        for i, player in enumerate(self.g.players):
            g_new = self.g.delete_action(i, action_to_delete)
            actions_to_remain = \
                np.setdiff1d(np.arange(player.num_actions), action_to_delete)
            assert_array_equal(
                g_new.payoff_profile_array,
                self.g.payoff_profile_array.take(actions_to_remain, axis=i)
            )

    def test_is_nash_pure(self):
        assert_(not self.g.is_nash((1, 0)))

    def test_is_nash_mixed(self):
        assert_(self.g.is_nash(([3/4, 1/4], [1/4, 3/4])))

    def test_payoff_arrays(self):
        assert_array_equal(
            self.g.payoff_arrays[0], self.BoS_bimatrix[:, :, 0]
        )
        assert_array_equal(
            self.g.payoff_arrays[1], self.BoS_bimatrix[:, :, 1].T
        )


class TestNormalFormGame_3p:
    """Test the methods of NormalFormGame with three players"""

    def setup_method(self):
        """Setup a NormalFormGame instance"""
        payoffs_2opponents = [[[3, 6],
                               [4, 2]],
                              [[1, 0],
                               [5, 7]]]
        player = Player(payoffs_2opponents)
        self.g = NormalFormGame([player for i in range(3)])

    def test_getitem(self):
        assert_array_equal(self.g[0, 0, 1], [6, 4, 1])

    def test_delete_action(self):
        action_to_delete = 0
        for i, player in enumerate(self.g.players):
            g_new = self.g.delete_action(i, action_to_delete)
            actions_to_remain = \
                np.setdiff1d(np.arange(player.num_actions), action_to_delete)
            assert_array_equal(
                g_new.payoff_profile_array,
                self.g.payoff_profile_array.take(actions_to_remain, axis=i)
            )

    def test_is_nash_pure(self):
        assert_(self.g.is_nash((0, 0, 0)))
        assert_(not self.g.is_nash((0, 0, 1)))

    def test_is_nash_mixed(self):
        p = (1 + np.sqrt(65)) / 16
        assert_(self.g.is_nash(([1 - p, p], [1 - p, p], [1 - p, p])))


def test_normalformgame_input_action_sizes():
    g = NormalFormGame((2, 3, 4))

    assert_(g.N == 3)  # Number of players

    assert_array_equal(
        g.players[0].payoff_array,
        np.zeros((2, 3, 4))
    )
    assert_array_equal(
        g.players[1].payoff_array,
        np.zeros((3, 4, 2))
    )
    assert_array_equal(
        g.players[2].payoff_array,
        np.zeros((4, 2, 3))
    )


def test_normalformgame_setitem():
    g = NormalFormGame((2, 2))
    g[0, 0] = (0, 10)
    g[0, 1] = (0, 10)
    g[1, 0] = (3, 5)
    g[1, 1] = (-2, 0)

    assert_array_equal(
        g.players[0].payoff_array,
        [[0, 0], [3, -2]]
    )
    assert_array_equal(
        g.players[1].payoff_array,
        [[10, 5], [10, 0]]
    )


def test_normalformgame_constant_payoffs():
    g = NormalFormGame((2, 2))

    assert_(g.is_nash((0, 0)))
    assert_(g.is_nash((0, 1)))
    assert_(g.is_nash((1, 0)))
    assert_(g.is_nash((1, 1)))


def test_normalformgame_payoff_profile_array():
    nums_actions = (2, 3, 4)
    for N in range(1, len(nums_actions)+1):
        payoff_arrays = [
            np.arange(np.prod(nums_actions[0:N])).reshape(nums_actions[i:N] +
                                                          nums_actions[0:i])
            for i in range(N)
        ]
        players = [Player(payoff_array) for payoff_array in payoff_arrays]
        g = NormalFormGame(players)
        g_new = NormalFormGame(g.payoff_profile_array)
        for player_new, payoff_array in zip(g_new.players, payoff_arrays):
            assert_array_equal(player_new.payoff_array, payoff_array)


def test_normalformgame_payoff_profile_array_c_contiguous():
    nums_actions = (2, 3, 4)
    shape = nums_actions + (len(nums_actions),)
    payoff_profile_array = \
        np.arange(np.prod(shape)).reshape(shape)
    g = NormalFormGame(payoff_profile_array)
    for player in g.players:
        assert_(player.payoff_array.flags['C_CONTIGUOUS'])


# Trivial cases with one player #

class TestPlayer_0opponents:
    """Test for trivial Player with no opponent player"""

    def setup_method(self):
        """Setup a Player instance"""
        self.payoffs = [0, 1, -1]
        self.player = Player(self.payoffs)
        self.best_response_action = 1
        self.dominated_actions = [0, 2]

    def test_delete_action(self):
        N = self.player.num_opponents + 1
        actions_to_delete = [0, 2]
        actions_to_remain = \
            np.setdiff1d(np.arange(self.player.num_actions), actions_to_delete)
        for i in range(N):
            player_new = self.player.delete_action(actions_to_delete, i)
            assert_array_equal(
                player_new.payoff_array,
                self.player.payoff_array.take(actions_to_remain, axis=i)
            )

    def test_payoff_vector(self):
        """Trivial player: payoff_vector"""
        assert_array_equal(self.player.payoff_vector(None), self.payoffs)

    def test_is_best_response(self):
        """Trivial player: is_best_response"""
        assert_(self.player.is_best_response(self.best_response_action, None))

    def test_best_response(self):
        """Trivial player: best_response"""
        assert_(self.player.best_response(None) == self.best_response_action)

    def test_is_dominated(self):
        """Trivial player: is_dominated"""
        for action in range(self.player.num_actions):
            assert_(self.player.is_dominated(action) ==
                    (action in self.dominated_actions))

    def test_dominated_actions(self):
        """Trivial player: dominated_actions"""
        assert_(self.player.dominated_actions() == self.dominated_actions)


class TestNormalFormGame_1p:
    """Test for trivial NormalFormGame with a single player"""

    def setup_method(self):
        """Setup a NormalFormGame instance"""
        data = [[0], [1], [1]]
        self.g = NormalFormGame(data)

    def test_construction(self):
        """Trivial game: construction"""
        assert_(self.g.N == 1)
        assert_array_equal(self.g.players[0].payoff_array, [0, 1, 1])

    def test_getitem(self):
        """Trivial game: __getitem__"""
        assert_(self.g[0] == 0)

    def test_delete_action(self):
        actions_to_delete = [1, 2]
        for i, player in enumerate(self.g.players):
            g_new = self.g.delete_action(i, actions_to_delete)
            actions_to_remain = \
                np.setdiff1d(np.arange(player.num_actions), actions_to_delete)
            assert_array_equal(
                g_new.payoff_profile_array,
                self.g.payoff_profile_array.take(actions_to_remain, axis=i)
            )

    def test_is_nash_pure(self):
        """Trivial game: is_nash with pure action"""
        assert_(self.g.is_nash((1,)))
        assert_(not self.g.is_nash((0,)))

    def test_is_nash_mixed(self):
        """Trivial game: is_nash with mixed action"""
        assert_(self.g.is_nash(([0, 1/2, 1/2],)))


def test_normalformgame_input_action_sizes_1p():
    g = NormalFormGame(2)

    assert_(g.N == 1)  # Number of players

    assert_array_equal(
        g.players[0].payoff_array,
        np.zeros(2)
    )


def test_normalformgame_setitem_1p():
    g = NormalFormGame(2)

    assert_(g.N == 1)  # Number of players

    g[0] = 10  # Set payoff 10 for action 0
    assert_(g.players[0].payoff_array[0] == 10)


# Trivial cases with one action #

class TestPlayer_1action:
    def setup_method(self):
        """Setup a Player instance"""
        self.payoffs = [[0, 1]]
        self.player = Player(self.payoffs)

    def test_is_dominated(self):
        for action in range(self.player.num_actions):
            for method in LP_METHODS:
                assert_(not self.player.is_dominated(action, method=method))

    def test_dominated_actions(self):
        for method in LP_METHODS:
            assert_(self.player.dominated_actions(method=method) == [])


# Test __repr__ #

def test_player_repr():
    nums_actions = (2, 3, 4)
    payoff_arrays = [
        np.arange(np.prod(nums_actions[0:i])).reshape(nums_actions[0:i])
        for i in range(1, len(nums_actions)+1)
    ]
    players = [Player(payoff_array) for payoff_array in payoff_arrays]

    for player in players:
        player_new = eval(repr(player))
        assert_array_equal(player_new.payoff_array, player.payoff_array)


# Invalid inputs #

def test_player_zero_actions():
    assert_raises(ValueError, Player, [[]])


def test_player_is_dominated_invalid_method():
    player = Player([[0, 0], [1, 1]])
    assert_raises(ValueError, player.is_dominated, 0, method='unknown method')


def test_normalformgame_invalid_input_players_shape_inconsistent():
    p0 = Player(np.zeros((2, 3)))
    p1 = Player(np.zeros((2, 3)))
    assert_raises(ValueError, NormalFormGame, [p0, p1])


def test_normalformgame_invalid_input_players_num_inconsistent():
    p0 = Player(np.zeros((2, 2, 2)))
    p1 = Player(np.zeros((2, 2, 2)))
    assert_raises(ValueError, NormalFormGame, [p0, p1])


def test_normalformgame_invalid_input_players_dtype_inconsistent():
    p0 = Player(np.zeros((2, 2), dtype=int))
    p1 = Player(np.zeros((2, 2), dtype=float))
    assert_raises(ValueError, NormalFormGame, [p0, p1])


def test_normalformgame_invalid_input_nosquare_matrix():
    assert_raises(ValueError, NormalFormGame, np.zeros((2, 3)))


def test_normalformgame_invalid_input_payoff_profiles():
    assert_raises(ValueError, NormalFormGame, np.zeros((2, 2, 1)))


def test_normalformgame_zero_actions():
    assert_raises(ValueError, NormalFormGame, (2, 0))


# Utility functions #

def test_pure2mixed():
    num_actions = 3
    pure_action = 0
    mixed_action = [1., 0., 0.]

    assert_array_equal(pure2mixed(num_actions, pure_action), mixed_action)


# Numba jitted functions #

def test_best_response_2p():
    test_case0 = {
        'payoff_array': np.array([[4, 0], [3, 2], [0, 3]]),
        'mixed_actions':
        [np.array([1, 0]), np.array([0.5, 0.5]), np.array([0, 1])],
        'brs_expected': [0, 1, 2]
    }
    test_case1 = {
        'payoff_array': np.zeros((2, 3)),
        'mixed_actions': [np.array([1, 0, 0]), np.array([1/3, 1/3, 1/3])],
        'brs_expected': [0, 0]
    }

    for test_case in [test_case0, test_case1]:
        for mixed_action, br_expected in zip(test_case['mixed_actions'],
                                             test_case['brs_expected']):
            br_computed = \
                best_response_2p(test_case['payoff_array'], mixed_action)
            assert_(br_computed == br_expected)



"""
Tests for gth_solve.py

"""
import numpy as np
from numpy.testing import (assert_array_equal, assert_raises, assert_,
                           assert_allclose)

from quantecon.markov import gth_solve
import pytest


TOL = 1e-15


def KMR_Markov_matrix_sequential(N, p, epsilon):
    """
    Generate the Markov matrix for the KMR model with *sequential* move

    Parameters
    ----------
    N : int
        Number of players

    p : float
        Level of p-dominance of action 1, i.e.,
        the value of p such that action 1 is the BR for (1-q, q) for any q > p,
        where q (1-q, resp.) is the prob that the opponent plays action 1 (0, resp.)

    epsilon : float
        Probability of mutation

    Returns
    -------
    P : numpy.ndarray
        Markov matrix for the KMR model with simultaneous move

    """
    P = np.zeros((N+1, N+1), dtype=float)
    P[0, 0], P[0, 1] = 1 - epsilon * (1/2), epsilon * (1/2)
    for n in range(1, N):
        P[n, n-1] = \
            (n/N) * (epsilon * (1/2) +
                     (1 - epsilon) * (((n-1)/(N-1) < p) + ((n-1)/(N-1) == p) * (1/2))
                     )
        P[n, n+1] = \
            ((N-n)/N) * (epsilon * (1/2) +
                         (1 - epsilon) * ((n/(N-1) > p) + (n/(N-1) == p) * (1/2))
                         )
        P[n, n] = 1 - P[n, n-1] - P[n, n+1]
    P[N, N-1], P[N, N] = epsilon * (1/2), 1 - epsilon * (1/2)
    return P


class Matrices:
    """Setup matrices for the tests"""

    def __init__(self):
        self.stoch_matrix_dicts = []
        self.kmr_matrix_dicts = []
        self.gen_matrix_dicts = []

        matrix_dict = {
            'A': np.array([[0.4, 0.6], [0.2, 0.8]]),
            'stationary_dist': np.array([0.25, 0.75]),
        }
        self.stoch_matrix_dicts.append(matrix_dict)

        matrix_dict = {
            # Reducible matrix
            'A': np.array([[1, 0], [0, 1]]),
            # Stationary dist whose support contains index 0
            'stationary_dist': np.array([1, 0]),
        }
        self.stoch_matrix_dicts.append(matrix_dict)

        matrix_dict = {
            'A': KMR_Markov_matrix_sequential(N=27, p=1./3, epsilon=1e-2),
        }
        self.kmr_matrix_dicts.append(matrix_dict)

        matrix_dict = {
            'A': KMR_Markov_matrix_sequential(N=3, p=1./3, epsilon=1e-14),
        }
        self.kmr_matrix_dicts.append(matrix_dict)

        matrix_dict = {
            'A': np.array([[-3, 3], [4, -4]]),
            'stationary_dist': np.array([4/7, 3/7]),
        }
        self.gen_matrix_dicts.append(matrix_dict)


class AddDescription:
    def __init__(self):
        self.description = self.__class__.__name__


class StationaryDistSumOne(AddDescription):
    def __call__(self, x):
        assert_allclose(np.sum(x), 1, atol=TOL)


class StationaryDistNonnegative(AddDescription):
    def __call__(self, x):
        assert_(np.prod(x >= 0-TOL) == 1)


class StationaryDistLeftEigenVec(AddDescription):
    def __call__(self, A, x):
        assert_allclose(np.dot(x, A), x, atol=TOL)


class StationaryDistEqualToKnown(AddDescription):
    def __call__(self, y, x):
        assert_allclose(y, x, atol=TOL)


test_classes = [
    StationaryDistSumOne,
    StationaryDistNonnegative,
]


@pytest.mark.parametrize("test_class", test_classes)
def test_stoch_matrix(test_class):
    """Test with stochastic matrices"""
    matrices = Matrices()
    for matrix_dict in matrices.stoch_matrix_dicts:
        x = gth_solve(matrix_dict['A'])
        test_class()(x)


def test_stoch_matrix_1():
    """Test with stochastic matrices"""
    matrices = Matrices()
    for matrix_dict in matrices.stoch_matrix_dicts:
        x = gth_solve(matrix_dict['A'])
        StationaryDistEqualToKnown()(matrix_dict['stationary_dist'], x)


@pytest.mark.parametrize("test_class", test_classes)
def test_kmr_matrix(test_class):
    """Test with KMR matrices"""
    matrices = Matrices()
    for matrix_dict in matrices.kmr_matrix_dicts:
        x = gth_solve(matrix_dict['A'])
        test_class()(x)


def test_kmr_matrix_1():
    """Test with KMR matrices"""
    matrices = Matrices()
    for matrix_dict in matrices.kmr_matrix_dicts:
        x = gth_solve(matrix_dict['A'])
        StationaryDistLeftEigenVec()(matrix_dict['A'], x)


@pytest.mark.parametrize("test_class", test_classes)
def test_gen_matrix(test_class):
    """Test with generator matrices"""
    matrices = Matrices()
    for matrix_dict in matrices.gen_matrix_dicts:
        x = gth_solve(matrix_dict['A'])
        test_class()(x)


def test_gen_matrix_1():
    """Test with generator matrices"""
    matrices = Matrices()
    for matrix_dict in matrices.gen_matrix_dicts:
        x = gth_solve(matrix_dict['A'])
        StationaryDistEqualToKnown()(matrix_dict['stationary_dist'], x)


def test_matrices_with_C_F_orders():
    """
    Test matrices with C- and F-contiguous orders
    See the issue and fix on Numba:
    github.com/numba/numba/issues/1103
    github.com/numba/numba/issues/1104

    Fix in gth_solve(A):
    added `order='C'` when `A1` copies the input `A`
    """
    P_C = np.array([[0.5, 0.5], [0, 1]], order='C')
    P_F = np.array([[0.5, 0.5], [0, 1]], order='F')
    stationary_dist = [0., 1.]

    computed_C_and_F = gth_solve(np.array([[1]]))
    assert_array_equal(computed_C_and_F, [1])

    computed_C = gth_solve(P_C)
    computed_F = gth_solve(P_F)
    assert_array_equal(computed_C, stationary_dist)
    assert_array_equal(computed_F, stationary_dist)


def test_raises_value_error_non_2dim():
    """Test with non 2dim input"""
    assert_raises(ValueError, gth_solve, np.array([0.4, 0.6]))


def test_raises_value_error_non_square():
    """Test with non square input"""
    assert_raises(ValueError, gth_solve, np.array([[0.4, 0.6]]))



import numpy as np
import numbers
from scipy import sparse
from ..util import check_random_state, rng_integers
from .random import random_pure_actions
from .normal_form_game import Player


class LocalInteraction:
    """
    Class representing the local interaction model.

    Parameters
    ----------
    payoff_matrix : array_like(float, ndim=2)
        The payoff matrix of the symmetric two-player game played in each
        interaction.

    adj_matrix : array_like(float, ndim=2)
        The adjacency matrix of the network. Non constant weights and asymmetry
        in interactions are allowed.

    Attributes
    ----------
    players : list(Player)
        The list consisting of all players with the given payoff matrix.

    adj_matrix : scipy.sparse.csr.csr_matrix(float, ndim=2)
        See Parameters.

    N : scalar(int)
        The number of players.

    num_actions : scalar(int)
        The number of actions available to each player.

    """
    def __init__(self, payoff_matrix, adj_matrix):
        self.adj_matrix = sparse.csr_matrix(adj_matrix)
        M, N = self.adj_matrix.shape
        if N != M:
            raise ValueError('adjacency matrix must be square')
        self.N = N

        A = np.asarray(payoff_matrix)
        if A.ndim != 2 or A.shape[0] != A.shape[1]:
            raise ValueError('payoff matrix must be square')
        self.num_actions = A.shape[0]

        self.players = [Player(A) for i in range(self.N)]
        self.tie_breaking = 'smallest'

    def _play(self, actions, player_ind, tie_breaking, tol, random_state):
        actions_matrix = sparse.csr_matrix(
            (np.ones(self.N, dtype=int), actions, np.arange(self.N+1)),
            shape=(self.N, self.num_actions))

        opponent_act_dict = self.adj_matrix[player_ind].dot(
            actions_matrix).toarray()

        for k, i in enumerate(player_ind):
            actions[i] = self.players[i].best_response(
                opponent_act_dict[k, :], tie_breaking=tie_breaking,
                tol=tol, random_state=random_state
            )

        return actions

    def play(self, revision='simultaneous', actions=None,
             player_ind_seq=None, num_reps=1, **options):
        """
        Return a new action profile which is updated by playing the game
        `num_reps` times.

        Parameters
        ----------
        revision : str, optional(default='simultaneous')
            The way to revise the action profile. If `simulataneous`, all
            players' actions will be updated simultaneously. If `asynchronous`,
            only designated players' actions will be updated. str in
            {'simultaneous', 'asynchronous'}.

        actions : tuple(int) or list(int), optional(default=None)
            The action profile in the first period. If None, selected randomly.

        player_ind_seq : array_like(int), optional(default=None)
            The index of players who take actions. If None, all players take
            actions.

        num_reps : scalar(int), optional(default=1)
            The number of iterations.

        **options : Keyword arguments passed to the best response method and
                    other methods.

        Returns
        -------
        tuple(int)
            The action profile after iterations.

        """
        tie_breaking = options.get('tie_breaking', self.tie_breaking)
        tol = options.get('tol', None)
        random_state = check_random_state(options.get('random_state', None))

        if actions is None:
            nums_actions = tuple([self.num_actions] * self.N)
            actions = random_pure_actions(nums_actions, random_state)

        if revision == 'simultaneous':
            player_ind_seq = [None] * num_reps
        elif revision == 'asynchronous':
            if player_ind_seq is None:
                random_state = check_random_state(random_state)
                player_ind_seq = rng_integers(random_state, self.N,
                                              size=num_reps)
            elif isinstance(player_ind_seq, numbers.Integral):
                player_ind_seq = [player_ind_seq]
        else:
            raise ValueError(
                            "revision must be `simultaneous` or `asynchronous`"
                            )

        actions = list(actions)
        for t, player_ind in enumerate(player_ind_seq):
            if player_ind is None:
                player_ind = list(range(self.N))
            elif isinstance(player_ind, numbers.Integral):
                player_ind = [player_ind]
            actions = self._play(actions, player_ind, tie_breaking,
                                 tol, random_state)

        return tuple(actions)

    def time_series(self, ts_length, revision='simultaneous',
                    actions=None, player_ind_seq=None, **options):
        """
        Return an array representing time series of each player's actions.

        Parameters
        ----------
        ts_length : scalar(int)
            The number of iterations.

        revision : {'simultaneous', 'asynchronous'}(default='simultaneous')
            The way to revise the action profile. If `simulataneous`, all
            players' actions will be updated simultaneously. If `asynchronous`,
            only designated players' actions will be updated. str in
            {'simultaneous', 'asynchronous'}.

        actions : tuple(int), optional(default=None)
            The action profile in the first period. If None, selected randomly.

        player_ind_seq : array_like, optional(default=None)
            The sequence of `player_ind`(see `play` Parameters) when the
            revision is 'asynchronous'. If None, selected randomly.

        **options : Keyword arguments passed to the best response method and
                    other methods.

        Returns
        -------
        Array_like(int)
            The array representing time series of each player's actions.

        """
        tie_breaking = options.get('tie_breaking', self.tie_breaking)
        tol = options.get('tol', None)
        random_state = check_random_state(options.get('random_state', None))

        if actions is None:
            nums_actions = tuple([self.num_actions] * self.N)
            actions = random_pure_actions(nums_actions, random_state)

        if revision == 'simultaneous':
            player_ind_seq = [None] * ts_length
        elif revision == 'asynchronous':
            if player_ind_seq is None:
                random_state = check_random_state(random_state)
                player_ind_seq = rng_integers(random_state, self.N,
                                              size=ts_length)
        else:
            raise ValueError(
                            "revision must be `simultaneous` or `asynchronous`"
                            )

        out = np.empty((ts_length, self.N), dtype=int)
        for i in range(self.N):
            out[0, i] = actions[i]
        for t in range(ts_length-1):
            random_state = check_random_state(random_state)
            actions = self.play(revision=revision,
                                actions=actions,
                                player_ind_seq=player_ind_seq[t],
                                num_reps=1,
                                tie_breaking=tie_breaking,
                                tol=tol, random_state=random_state)
            for i in range(self.N):
                out[t+1, i] = actions[i]

        return out



import numpy as np
from numba import jit
from .utilities import _find_indices
from ..optimize.linprog_simplex import solve_tableau, PivOptions
from ..optimize.pivoting import _pivoting


@jit(nopython=True, cache=True)
def ddp_linprog_simplex(R, Q, beta, a_indices, a_indptr, sigma,
                        max_iter=10**6, piv_options=PivOptions(),
                        tableau=None, basis=None, v=None):
    r"""
    Numba jit complied function to solve a discrete dynamic program via
    linear programming, using `optimize.linprog_simplex` routines. The
    problem has to be represented in state-action pair form with 1-dim
    reward ndarray `R` of shape (n,), 2-dim transition probability
    ndarray `Q` of shapce (L, n), and disount factor `beta`, where n is
    the number of states and L is the number of feasible state-action
    pairs.

    The approach exploits the fact that the optimal value function is
    the smallest function that satisfies :math:`v \geq T v`, where
    :math:`T` is the Bellman operator, and hence it is a (unique)
    solution to the linear program:

    minimize::

        \sum_{s \in S} v(s)

    subject to ::

        v(s) \geq r(s, a) + \beta \sum_{s' \in S} q(s'|s, a) v(s')
                  \quad ((s, a) \in \mathit{SA}).

    This function solves its dual problem:

    maximize::

        \sum_{(s, a) \in \mathit{SA}} r(s, a) y(s, a)

    subject to::

        \sum_{a: (s', a) \in \mathit{SA}} y(s', a) -
            \sum_{(s, a) \in \mathit{SA}} \beta q(s'|s, a) y(s, a) = 1
            \quad (s' \in S),

        y(s, a) \geq 0 \quad ((s, a) \in \mathit{SA}),

    where the optimal value function is obtained as an optimal dual
    solution and an optimal policy as an optimal basis.

    Parameters
    ----------
    R : ndarray(float, ndim=1)
        Reward ndarray, of shape (n,).

    Q : ndarray(float, ndim=2)
        Transition probability ndarray, of shape (L, n).

    beta : scalar(float)
        Discount factor. Must be in [0, 1).

    a_indices : ndarray(int, ndim=1)
        Action index ndarray, of shape (L,).

    a_indptr : ndarray(int, ndim=1)
        Action index pointer ndarray, of shape (n+1,).

    sigma : ndarray(int, ndim=1)
        ndarray containing the initial feasible policy, of shape (n,).
        To be modified in place to store the output optimal policy.

    max_iter : int, optional(default=10**6)
        Maximum number of iteration in the linear programming solver.

    piv_options : PivOptions, optional
        PivOptions namedtuple to set tolerance values used in the linear
        programming solver.

    tableau : ndarray(float, ndim=2), optional
        Temporary ndarray of shape (n+1, L+n+1) to store the tableau.
        Modified in place.

    basis : ndarray(int, ndim=1), optional
        Temporary ndarray of shape (n,) to store the basic variables.
        Modified in place.

    v : ndarray(float, ndim=1), optional
        Output ndarray of shape (n,) to store the optimal value
        function. Modified in place.

    Returns
    -------
    success : bool
        True if the algorithm succeeded in finding an optimal solution.

    num_iter : int
        The number of iterations performed.

    v : ndarray(float, ndim=1)
        Optimal value function (view to `v` if supplied).

    sigma : ndarray(int, ndim=1)
        Optimal policy (view to `sigma`).

    """
    L, n = Q.shape

    if tableau is None:
        tableau = np.empty((n+1, L+n+1))
    if basis is None:
        basis = np.empty(n, dtype=np.int_)
    if v is None:
        v = np.empty(n)

    _initialize_tableau(R, Q, beta, a_indptr, tableau)
    _find_indices(a_indices, a_indptr, sigma, out=basis)

    # Phase 1
    for i in range(n):
        _pivoting(tableau, basis[i], i)

    # Phase 2
    success, status, num_iter = \
        solve_tableau(tableau, basis, max_iter-n, skip_aux=True,
                      piv_options=piv_options)

    # Obtain solution
    for i in range(n):
        v[i] = tableau[-1, L+i] * (-1)

    for i in range(n):
        sigma[i] = a_indices[basis[i]]

    return success, num_iter+n, v, sigma


@jit(nopython=True, cache=True)
def _initialize_tableau(R, Q, beta, a_indptr, tableau):
    """
    Initialize the `tableau` array.

    Parameters
    ----------
    R : ndarray(float, ndim=1)
        Reward ndarray, of shape (n,).

    Q : ndarray(float, ndim=2)
        Transition probability ndarray, of shape (L, n).

    beta : scalar(float)
        Discount factor. Must be in [0, 1).

    a_indptr : ndarray(int, ndim=1)
        Action index pointer ndarray, of shape (n+1,).

    tableau : ndarray(float, ndim=2)
        Empty ndarray of shape (n+1, L+n+1) to store the tableau.
        Modified in place.

    Returns
    -------
    tableau : ndarray(float, ndim=2)
        View to `tableau`.

    """
    L, n = Q.shape

    for j in range(L):
        for i in range(n):
            tableau[i, j] = Q[j, i] * (-beta)

    for i in range(n):
        for j in range(a_indptr[i], a_indptr[i+1]):
            tableau[i, j] += 1

    tableau[:n, L:-1] = 0

    for i in range(n):
        tableau[i, L+i] = 1
        tableau[i, -1] = 1

    for j in range(L):
        tableau[-1, j] = R[j]

    tableau[-1, L:] = 0

    return tableau



"""
Tests for vertex_enumeration.py

"""
import numpy as np
from numpy.testing import (assert_allclose, assert_array_equal, assert_,
                           assert_raises)
from quantecon.game_theory import NormalFormGame, vertex_enumeration
from quantecon.game_theory.vertex_enumeration import _BestResponsePolytope


class TestVertexEnumeration:
    def setup_method(self):
        self.game_dicts = []

        # From von Stengel 2007 in Algorithmic Game Theory
        bimatrix = [[(3, 3), (3, 2)],
                    [(2, 2), (5, 6)],
                    [(0, 3), (6, 1)]]
        d = {'g': NormalFormGame(bimatrix),
             'NEs': [([1, 0, 0], [1, 0]),
                     ([4/5, 1/5, 0], [2/3, 1/3]),
                     ([0, 1/3, 2/3], [1/3, 2/3])]}
        self.game_dicts.append(d)

        # Degenerate game
        bimatrix = [[(3, 3), (3, 3)],
                    [(2, 2), (5, 6)],
                    [(0, 3), (6, 1)]]
        d = {'g': NormalFormGame(bimatrix),
             'NEs': [([1, 0, 0], [1, 0]),
                     ([1, 0, 0], [2/3, 1/3]),
                     ([0, 1/3, 2/3], [1/3, 2/3])]}
        self.game_dicts.append(d)

    def test_vertex_enumeration(self):
        for d in self.game_dicts:
            NEs_computed = vertex_enumeration(d['g'])
            assert_(len(NEs_computed) == len(d['NEs']))
            for NEs in (NEs_computed, d['NEs']):
                NEs.sort(key=lambda x: (list(x[0]), list(x[1])))
            for actions_computed, actions in zip(NEs_computed, d['NEs']):
                for action_computed, action in zip(actions_computed, actions):
                    assert_allclose(action_computed, action)


def test_vertex_enumeration_qhull_options():
    # Degenerate game, player 0's actions reordered
    bimatrix = [[(0, 3), (6, 1)],
                [(2, 2), (5, 6)],
                [(3, 3), (3, 3)]]
    g = NormalFormGame(bimatrix)
    NEs_expected = [([0, 0, 1], [1, 0]),
                    ([0, 0, 1], [2/3, 1/3]),
                    ([2/3, 1/3, 0], [1/3, 2/3])]
    qhull_options = 'QJ'
    NEs_computed = vertex_enumeration(g, qhull_options=qhull_options)
    assert_(len(NEs_computed) == len(NEs_expected))
    for NEs in (NEs_computed, NEs_expected):
        NEs.sort(key=lambda x: (list(x[1]), list(x[0])))
    for actions_computed, actions in zip(NEs_computed, NEs_expected):
        for action_computed, action in zip(actions_computed, actions):
            assert_allclose(action_computed, action, atol=1e-10)


def test_vertex_enumeration_invalid_g():
    bimatrix = [[(3, 3), (3, 2)],
                [(2, 2), (5, 6)],
                [(0, 3), (6, 1)]]
    assert_raises(TypeError, vertex_enumeration, bimatrix)


class TestBestResponsePolytope:
    def setup_method(self):
        # From von Stengel 2007 in Algorithmic Game Theory
        bimatrix = [[(3, 3), (3, 2)],
                    [(2, 2), (5, 6)],
                    [(0, 3), (6, 1)]]
        g = NormalFormGame(bimatrix)

        # Original best response polytope for player 0
        vertices_P = np.array([
            [0, 0, 0],       # 0
            [1/3, 0, 0],     # a
            [2/7, 1/14, 0],  # b
            [0, 1/6, 0],     # c
            [0, 1/8, 1/4],   # d
            [0, 0, 1/3]      # e
        ])
        labelings_P = np.array([
            [0, 1, 2],  # 0
            [1, 2, 3],  # a
            [2, 3, 4],  # b
            [0, 2, 4],  # c
            [0, 3, 4],  # d
            [0, 1, 3]   # e
        ])

        # Sort rows lexicographically
        K = labelings_P.shape[1]
        ind = np.lexsort([labelings_P[:, K-k-1] for k in range(K)])
        self.labelings_P = labelings_P[ind]
        self.vertices_P = vertices_P[ind]

        # Translated best response polytope for player 0
        self.brp0 = _BestResponsePolytope(g.players[1], idx=0)

    def test_best_response_polytope(self):
        # Sort each row
        labelings_computed = np.sort(self.brp0.labelings, axis=1)

        # Sort rows lexicographically
        K = labelings_computed.shape[1]
        ind = np.lexsort([labelings_computed[:, K-k-1] for k in range(K)])
        labelings_computed = labelings_computed[ind]
        vertices_computed = \
            -self.brp0.equations[:, :-1] / self.brp0.equations[:, [-1]] + \
            1/self.brp0.trans_recip
        vertices_computed = vertices_computed[ind]

        assert_array_equal(labelings_computed, self.labelings_P)
        assert_allclose(vertices_computed, self.vertices_P, atol=1e-15)


def test_best_response_polytope_invalid_player_instance():
    bimatrix = [[(3, 3), (3, 2)],
                [(2, 2), (5, 6)],
                [(0, 3), (6, 1)]]
    g = NormalFormGame(bimatrix)
    assert_raises(TypeError, _BestResponsePolytope, g)



"""
Tests for compute_fp.py

References
----------

https://www.math.ucdavis.edu/~hunter/book/ch3.pdf

TODO: add multivariate case

"""
import numpy as np
from numpy.testing import assert_, assert_raises
from quantecon import compute_fixed_point


class TestFPLogisticEquation():

    @classmethod
    def setup_method(cls):
        cls.mu_1 = 0.2  # 0 is unique fixed point forall x_0 \in [0, 1]

        # (4mu - 1)/(4mu) is a fixed point forall x_0 \in [0, 1]
        cls.mu_2 = 0.3

        # starting points on (0, 1)
        cls.unit_inverval = [0.1, 0.3, 0.6, 0.9]

        # arguments for compute_fixed_point
        cls.kwargs = {"error_tol": 1e-5, "max_iter": 200, "verbose": 0}

    def T(self, x, mu):
        return 4.0 * mu * x * (1.0 - x)

    def test_contraction_1(self):
        "compute_fp: convergence inside interval of convergence"
        f = lambda x: self.T(x, self.mu_1)
        for i in self.unit_inverval:
            # should have fixed point of 0.0
            assert_(abs(compute_fixed_point(f, i, **self.kwargs)) < 1e-4)

    def test_not_contraction_2(self):
        "compute_fp: no convergence outside interval of convergence"
        f = lambda x: self.T(x, self.mu_2)
        for i in self.unit_inverval:
            # This shouldn't converge to 0.0
            assert_(not (abs(compute_fixed_point(f, i, **self.kwargs)) < 1e-4))

    def test_contraction_2(self):
        "compute_fp: convergence inside interval of convergence"
        f = lambda x: self.T(x, self.mu_2)
        fp = (4 * self.mu_2 - 1) / (4 * self.mu_2)
        for i in self.unit_inverval:
            # This should converge to fp
            assert_(abs(compute_fixed_point(f, i, **self.kwargs)-fp) < 1e-4)

    def test_not_contraction_1(self):
        "compute_fp: no convergence outside interval of convergence"
        f = lambda x: self.T(x, self.mu_1)
        fp = (4 * self.mu_1 - 1) / (4 * self.mu_1)
        for i in self.unit_inverval:
            # This should not converge  (b/c unique fp is 0.0)
            assert_(not (abs(compute_fixed_point(f, i, **self.kwargs)-fp)
                    < 1e-4))

    def test_imitation_game_method(self):
        "compute_fp: Test imitation game method"
        method = 'imitation_game'
        error_tol = self.kwargs['error_tol']

        for mu in [self.mu_1, self.mu_2]:
            for i in self.unit_inverval:
                fp_computed = compute_fixed_point(self.T, i, method=method,
                                                  mu=mu, **self.kwargs)
                assert_(
                    abs(self.T(fp_computed, mu=mu) - fp_computed) <= error_tol
                )

            # numpy array input
            i = np.asarray(self.unit_inverval)
            fp_computed = compute_fixed_point(self.T, i, method=method, mu=mu,
                                              **self.kwargs)
            assert_(
                abs(self.T(fp_computed, mu=mu) - fp_computed).max() <=
                error_tol
            )


class TestComputeFPContraction():
    def setup_method(self):
        self.coeff = 0.5
        self.methods = ['iteration', 'imitation_game']

    def f(self, x):
        return self.coeff * x

    def test_num_iter_one(self):
        init = 1.
        error_tol = self.coeff

        for method in self.methods:
            fp_computed = compute_fixed_point(self.f, init,
                                              error_tol=error_tol,
                                              method=method)
            assert_(fp_computed <= error_tol * 2)

    def test_num_iter_large(self):
        init = 1.
        buff_size = 2**8  # buff_size in 'imitation_game'
        max_iter = buff_size + 2
        error_tol = self.coeff**max_iter

        for method in self.methods:
            fp_computed = compute_fixed_point(self.f, init,
                                              error_tol=error_tol,
                                              max_iter=max_iter, method=method,
                                              print_skip=max_iter)
            assert_(fp_computed <= error_tol * 2)

    def test_2d_input(self):
        error_tol = self.coeff**4

        for method in self.methods:
            init = np.array([[-1, 0.5], [-1/3, 0.1]])
            fp_computed = compute_fixed_point(self.f, init,
                                              error_tol=error_tol,
                                              method=method)
            assert_((fp_computed <= error_tol * 2).all())


def test_raises_value_error_nonpositive_max_iter():
    f = lambda x: 0.5*x
    init = 1.
    max_iter = 0
    assert_raises(ValueError, compute_fixed_point, f, init, max_iter=max_iter)



"""
Warnings Module
===============

Contains a collection of warning messages for consistent package wide notifications

"""

#-Numba-#
numba_import_fail_message = ("Numba import failed. Falling back to non-optimized routines.\n" 
                            "This will reduce the overall performance of this package.\n"    
                            "To install please use the anaconda distribution.\n"             
                            "http://continuum.io/downloads")



"""
Contain pivoting routines commonly used in the Simplex Algorithm and
Lemke-Howson Algorithm routines.

"""
import numpy as np
from numba import jit


TOL_PIV = 1e-10
TOL_RATIO_DIFF = 1e-15


@jit(nopython=True, cache=True)
def _pivoting(tableau, pivot_col, pivot_row):
    """
    Perform a pivoting step. Modify `tableau` in place.

    Parameters
    ----------
    tableau : ndarray(float, ndim=2)
        Array containing the tableau.

    pivot_col : scalar(int)
        Pivot column index.

    pivot_row : scalar(int)
        Pivot row index.

    Returns
    -------
    tableau : ndarray(float, ndim=2)
        View to `tableau`.

    """
    nrows, ncols = tableau.shape

    pivot_elt = tableau[pivot_row, pivot_col]
    for j in range(ncols):
        tableau[pivot_row, j] /= pivot_elt

    for i in range(nrows):
        if i == pivot_row:
            continue
        multiplier = tableau[i, pivot_col]
        if multiplier == 0:
            continue
        for j in range(ncols):
            tableau[i, j] -= tableau[pivot_row, j] * multiplier

    return tableau


@jit(nopython=True, cache=True)
def _min_ratio_test_no_tie_breaking(tableau, pivot, test_col,
                                    argmins, num_candidates,
                                    tol_piv, tol_ratio_diff):
    """
    Perform the minimum ratio test, without tie breaking, for the
    candidate rows in `argmins[:num_candidates]`. Return the number
    `num_argmins` of the rows minimizing the ratio and store thier
    indices in `argmins[:num_argmins]`.

    Parameters
    ----------
    tableau : ndarray(float, ndim=2)
        Array containing the tableau.

    pivot : scalar(int)
        Pivot.

    test_col : scalar(int)
        Index of the column used in the test.

    argmins : ndarray(int, ndim=1)
        Array containing the indices of the candidate rows. Modified in
        place to store the indices of minimizing rows.

    num_candidates : scalar(int)
        Number of candidate rows in `argmins`.

    tol_piv : scalar(float)
        Pivot tolerance below which a number is considered to be
        nonpositive.

    tol_ratio_diff : scalar(float)
        Tolerance to determine a tie between ratio values.

    Returns
    -------
    num_argmins : scalar(int)
        Number of minimizing rows.

    """
    ratio_min = np.inf
    num_argmins = 0

    for k in range(num_candidates):
        i = argmins[k]
        if tableau[i, pivot] <= tol_piv:  # Treated as nonpositive
            continue
        ratio = tableau[i, test_col] / tableau[i, pivot]
        if ratio > ratio_min + tol_ratio_diff:  # Ratio large for i
            continue
        elif ratio < ratio_min - tol_ratio_diff:  # Ratio smaller for i
            ratio_min = ratio
            num_argmins = 1
        else:  # Ratio equal
            num_argmins += 1
        argmins[num_argmins-1] = i

    return num_argmins


@jit(nopython=True, cache=True)
def _lex_min_ratio_test(tableau, pivot, slack_start, argmins,
                        tol_piv=TOL_PIV, tol_ratio_diff=TOL_RATIO_DIFF):
    """
    Perform the lexico-minimum ratio test.

    Parameters
    ----------
    tableau : ndarray(float, ndim=2)
        Array containing the tableau.

    pivot : scalar(int)
        Pivot.

    slack_start : scalar(int)
        First index for the slack variables.

    argmins : ndarray(int, ndim=1)
        Empty array used to store the row indices. Its length must be no
        smaller than the number of the rows of `tableau`.

    tol_piv : scalar(float), optional
        Pivot tolerance below which a number is considered to be
        nonpositive. Default value is {TOL_PIV}.

    tol_ratio_diff : scalar(float), optional
        Tolerance to determine a tie between ratio values. Default value
        is {TOL_RATIO_DIFF}.

    Returns
    -------
    found : bool
        False if there is no positive entry in the pivot column.

    row_min : scalar(int)
        Index of the row with the lexico-minimum ratio.

    """
    nrows = tableau.shape[0]
    num_candidates = nrows

    found = False

    # Initialize `argmins`
    for i in range(nrows):
        argmins[i] = i

    num_argmins = _min_ratio_test_no_tie_breaking(
        tableau, pivot, -1, argmins, num_candidates, tol_piv, tol_ratio_diff
    )
    if num_argmins == 1:
        found = True
    elif num_argmins >= 2:
        for j in range(slack_start, slack_start+nrows):
            if j == pivot:
                continue
            num_argmins = _min_ratio_test_no_tie_breaking(
                tableau, pivot, j, argmins, num_argmins,
                tol_piv, tol_ratio_diff
            )
            if num_argmins == 1:
                found = True
                break
    return found, argmins[0]


_lex_min_ratio_test.__doc__ = _lex_min_ratio_test.__doc__.format(
    TOL_PIV=TOL_PIV, TOL_RATIO_DIFF=TOL_RATIO_DIFF
)



"""
Filename: test_fictplay.py

Tests for fictplay.py

"""
import numpy as np
from numpy.testing import assert_array_almost_equal
from scipy.stats import norm

from quantecon.game_theory import FictitiousPlay, StochasticFictitiousPlay


class TestFictitiousPlayDecreaingGain:

    def setup_method(self):
        '''Setup a FictitiousPlay instance'''
        # symmetric 2x2 coordination game
        matching_pennies = [[(1, -1), (-1, 1)],
                            [(-1, 1), (1, -1)]]
        self.fp = FictitiousPlay(matching_pennies)

    def test_play(self):
        x = (np.array([1, 0]), np.array([0.5, 0.5]))
        assert_array_almost_equal(self.fp.play(actions=(0, 0)), x)

    def test_time_series(self):
        x = self.fp.time_series(ts_length=3, init_actions=(0, 0))
        assert_array_almost_equal(x[0], [[1, 0],
                                         [1, 0],
                                         [1, 0]])
        assert_array_almost_equal(x[1], [[1, 0],
                                         [1/2, 1/2],
                                         [1/3, 2/3]])


class TestFictitiousPlayConstantGain:

    def setup_method(self):
        matching_pennies = [[(1, -1), (-1, 1)],
                            [(-1, 1), (1, -1)]]
        self.fp = FictitiousPlay(matching_pennies, gain=0.1)

    def test_play(self):
        x = (np.array([1, 0]), np.array([0.9, 0.1]))
        assert_array_almost_equal(self.fp.play(actions=(0, 0)), x)

    def test_time_series(self):
        x = self.fp.time_series(ts_length=3, init_actions=(0, 0))
        assert_array_almost_equal(x[0], [[1, 0],
                                         [1, 0],
                                         [1, 0]])
        assert_array_almost_equal(x[1], [[1, 0],
                                         [0.9, 0.1],
                                         [0.81, 0.19]])


class TestStochasticFictitiosuPlayDecreaingGain:

    def setup_method(self):
        matching_pennies = [[(1, -1), (-1, 1)],
                            [(-1, 1), (1, -1)]]
        distribution = norm()
        self.fp = StochasticFictitiousPlay(matching_pennies,
                                           distribution=distribution)

    def test_play(self):
        seed = 272733541340907175684079858751241831341
        x = [self.fp.play(actions=(0, 0),
                          random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_almost_equal(x[0], x[1])

    def test_time_series(self):
        seed = 226177486389088886197048956835604946950
        x = [self.fp.time_series(ts_length=3, init_actions=(0, 0),
                                 random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_almost_equal(x[0][0], x[1][0])
        assert_array_almost_equal(x[0][1], x[1][1])


class TestStochasticFictitiosuPlayConstantGain:

    def setup_method(self):
        matching_pennies = [[(1, -1), (-1, 1)],
                            [(-1, 1), (1, -1)]]
        distribution = norm()
        self.fp = StochasticFictitiousPlay(matching_pennies, gain=0.1,
                                           distribution=distribution)

    def test_play(self):
        seed = 271001177347704493622442691590340912076
        x = [self.fp.play(actions=(0, 0),
                          random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_almost_equal(x[0], x[1])

    def test_time_series(self):
        seed = 143773081180220547556482766921740826832
        x = [self.fp.time_series(ts_length=3, init_actions=(0, 0),
                                 random_state=np.random.default_rng(seed))
             for i in range(2)]
        assert_array_almost_equal(x[0][0], x[1][0])
        assert_array_almost_equal(x[0][1], x[1][1])
